@@ -18,9 +18,9 @@ from oslo_config import cfg
 from oslo_log import log
 from oslo_utils import importutils
 
+from keystoneauth1.identity import v3 as ks_identity_v3
 from keystoneauth1 import session as ks_session
 from keystoneauth1.token_endpoint import Token
-from keystoneclient.auth import identity
 from keystoneclient import httpclient
 
 from mistral.actions.openstack import base
@@ -52,7 +52,8 @@ def _try_import(module_name):
 aodhclient = _try_import('aodhclient.v2.client')
 barbicanclient = _try_import('barbicanclient.client')
 cinderclient = _try_import('cinderclient.client')
-designateclient = _try_import('designateclient.v1')
+cinder_api_versions = _try_import('cinderclient.api_versions')
+designateclient = _try_import('designateclient.v2.client')
 glanceclient = _try_import('glanceclient')
 glareclient = _try_import('glareclient.v1.client')
 gnocchiclient = _try_import('gnocchiclient.v1.client')
@@ -67,7 +68,9 @@ magnumclient = _try_import('magnumclient.v1.client')
 mistralclient = _try_import('mistralclient.api.v2.client')
 muranoclient = _try_import('muranoclient.v1.client')
 neutronclient = _try_import('neutronclient.v2_0.client')
+nova = _try_import('novaclient')
 novaclient = _try_import('novaclient.client')
+nova_api_versions = _try_import('novaclient.api_versions')
 qinlingclient = _try_import('qinlingclient.v1.client')
 senlinclient = _try_import('senlinclient.v1.client')
 swift_client = _try_import('swiftclient.client')
@@ -77,6 +80,7 @@ troveclient = _try_import('troveclient.v1.client')
 vitrageclient = _try_import('vitrageclient.v1.client')
 zaqarclient = _try_import('zaqarclient.queues.client')
 zunclient = _try_import('zunclient.v1.client')
+zun_api_versions = _try_import('zunclient.api_versions')
 
 
 class NovaAction(base.OpenStackAction):
@@ -93,8 +97,19 @@ class NovaAction(base.OpenStackAction):
 
         session_and_auth = self.get_session_and_auth(context)
 
-        return novaclient.Client(
-            2,
+        temp_client = self._get_client_class()(
+            nova.API_MAX_VERSION,
+            endpoint_override=nova_endpoint.url,
+            session=session_and_auth['session']
+        )
+
+        discovered_version = nova_api_versions.discover_version(
+            temp_client,
+            nova.API_MAX_VERSION
+        )
+
+        return self._get_client_class()(
+            discovered_version,
             endpoint_override=nova_endpoint.url,
             session=session_and_auth['session']
         )
@@ -220,7 +235,7 @@ class NeutronAction(base.OpenStackAction):
 
 
 class CinderAction(base.OpenStackAction):
-    _service_type = 'volumev2'
+    _service_type = 'volumev3'
 
     @classmethod
     def _get_client_class(cls):
@@ -234,17 +249,26 @@ class CinderAction(base.OpenStackAction):
 
         session_and_auth = self.get_session_and_auth(context)
 
-        client = self._get_client_class()(
-            '2',
+        temp_client = self._get_client_class()(
+            cinder_api_versions.MAX_VERSION,
             endpoint_override=cinder_endpoint.url,
             session=session_and_auth['session']
         )
 
-        return client
+        discovered_version = cinder_api_versions.discover_version(
+            temp_client,
+            cinder_api_versions.APIVersion(cinder_api_versions.MAX_VERSION),
+        )
+
+        return self._get_client_class()(
+            discovered_version,
+            endpoint_override=cinder_endpoint.url,
+            session=session_and_auth['session']
+        )
 
     @classmethod
     def _get_fake_client(cls):
-        return cls._get_client_class()('2')
+        return cls._get_client_class()('3')
 
 
 class MistralAction(base.OpenStackAction):
@@ -324,17 +348,22 @@ class IronicAction(base.OpenStackAction):
 
         ironic_endpoint = self.get_service_endpoint()
 
-        return self._get_client_class()(
-            ironic_endpoint.url,
-            token=context.auth_token,
-            region_name=ironic_endpoint.region,
+        session_and_auth = self.get_session_and_auth(context)
+
+        client = self._get_client_class()(
             os_ironic_api_version=IRONIC_API_VERSION,
-            insecure=context.insecure
+            endpoint_override=ironic_endpoint.url,
+            session=session_and_auth['session']
         )
 
+        return client
+
     @classmethod
     def _get_fake_client(cls):
-        return cls._get_client_class()("http://127.0.0.1:6385/")
+        return cls._get_client_class()(
+            endpoint_override="http://127.0.0.1:6385/",
+            session=ks_session.Session(),
+        )
 
 
 class BaremetalIntrospectionAction(base.OpenStackAction):
@@ -366,7 +395,6 @@ class BaremetalIntrospectionAction(base.OpenStackAction):
             return cls._get_client_class()()
 
     def _create_client(self, context):
-
         LOG.debug(
             "Baremetal introspection action security context: %s", context)
 
@@ -445,7 +473,6 @@ class ZaqarAction(base.OpenStackAction):
         return zaqarclient.Client
 
     def _create_client(self, context):
-
         LOG.debug("Zaqar action security context: %s", context)
 
         zaqar_endpoint = self.get_service_endpoint()
@@ -574,30 +601,21 @@ class ZaqarAction(base.OpenStackAction):
 
 
 class BarbicanAction(base.OpenStackAction):
+    _service_type = 'key-manager'
 
     @classmethod
     def _get_client_class(cls):
         return barbicanclient.Client
 
     def _create_client(self, context):
-
         LOG.debug("Barbican action security context: %s", context)
 
-        barbican_endpoint = keystone_utils.get_endpoint_for_project('barbican')
-        keystone_endpoint = keystone_utils.get_keystone_endpoint()
-
-        auth = identity.v3.Token(
-            auth_url=keystone_endpoint.url,
-            project_name=context.user_name,
-            token=context.auth_token,
-            project_id=context.project_id
-        )
+        barbican_endpoint = self.get_service_endpoint()
+        session_and_auth = self.get_session_and_auth(context)
 
         return self._get_client_class()(
-            project_id=context.project_id,
             endpoint=barbican_endpoint.url,
-            auth=auth,
-            insecure=context.insecure
+            session=session_and_auth['session']
         )
 
     @classmethod
@@ -609,7 +627,8 @@ class BarbicanAction(base.OpenStackAction):
 
     @classmethod
     def _get_client_method(cls, client):
-        if cls.client_method_name != "secrets_store":
+        if cls.client_method_name not in ["secrets_store",
+                                          "secrets_retrieve"]:
             return super(BarbicanAction, cls)._get_client_method(client)
 
         method = getattr(cls, cls.client_method_name)
@@ -635,8 +654,8 @@ class BarbicanAction(base.OpenStackAction):
                       mode=None, expiration=None):
         """Create and Store a secret in Barbican.
 
-        :param client: the Zaqar client
-        :type client: zaqarclient.queues.client
+        :param client: the Barbican client
+        :type client: barbicanclient.client
 
         :param name: A friendly name for the Secret
         :type name: string
@@ -677,6 +696,19 @@ class BarbicanAction(base.OpenStackAction):
 
         return entity._get_formatted_entity()
 
+    @staticmethod
+    def secrets_retrieve(client, secret_ref):
+        """Retrieve the payload from a secret in Barbican.
+
+        :param client: the Barbican client
+        :type client: barbicanclient.client
+
+        :param secret_ref: Full HATEOAS reference to a Secret
+        :type name: string
+        """
+
+        return client.secrets.get(secret_ref).payload
+
 
 class DesignateAction(base.OpenStackAction):
     _service_type = 'dns'
@@ -686,28 +718,14 @@ class DesignateAction(base.OpenStackAction):
         return designateclient.Client
 
     def _create_client(self, context):
-
         LOG.debug("Designate action security context: %s", context)
 
-        designate_endpoint = self.get_service_endpoint()
-
-        designate_url = keystone_utils.format_url(
-            designate_endpoint.url,
-            {'tenant_id': context.project_id}
-        )
+        session_and_auth = self.get_session_and_auth(context)
 
         client = self._get_client_class()(
-            endpoint=designate_url,
-            tenant_id=context.project_id,
-            auth_url=context.auth_uri,
-            region_name=designate_endpoint.region,
-            service_type='dns',
-            insecure=context.insecure
+            session=session_and_auth['session']
         )
 
-        client.client.auth_token = context.auth_token
-        client.client.management_url = designate_url
-
         return client
 
     @classmethod
@@ -812,9 +830,30 @@ class SenlinAction(base.OpenStackAction):
         keystone_endpoint = keystone_utils.get_keystone_endpoint()
         senlin_endpoint = self.get_service_endpoint()
 
+        if context.is_trust_scoped and keystone_utils.is_token_trust_scoped(
+                context.auth_token):
+            if context.trust_id is None:
+                raise Exception(
+                    "'trust_id' must be provided in the admin context."
+                )
+
+            auth = ks_identity_v3.Password(
+                auth_url=keystone_endpoint.url,
+                trust_id=context.trust_id,
+                username=CONF.keystone_authtoken.username,
+                password=CONF.keystone_authtoken.password,
+                user_domain_name=CONF.keystone_authtoken.user_domain_name
+            )
+        else:
+            auth = ks_identity_v3.Token(
+                auth_url=keystone_endpoint.url,
+                token=context.auth_token,
+                project_id=context.project_id
+            )
+
         return self._get_client_class()(
             endpoint_url=senlin_endpoint.url,
-            token=context.auth_token,
+            session=ks_session.Session(auth=auth),
             tenant_id=context.project_id,
             region_name=senlin_endpoint.region,
             auth_url=keystone_endpoint.url,
@@ -823,8 +862,9 @@ class SenlinAction(base.OpenStackAction):
 
     @classmethod
     def _get_fake_client(cls):
-        sess = keystone_utils.get_admin_session()
-        return cls._get_client_class()(authenticator=sess.auth)
+        # Senlin client changed interface a bit, let's skip __init__ altogether
+        class_ = cls._get_client_class()
+        return class_.__new__(class_)
 
 
 class AodhAction(base.OpenStackAction):
@@ -961,15 +1001,25 @@ class ZunAction(base.OpenStackAction):
         zun_endpoint = self.get_service_endpoint()
         session_and_auth = self.get_session_and_auth(context)
 
-        client = self._get_client_class()(
-            '1',
+        temp_client = self._get_client_class()(
+            zun_api_versions.MAX_API_VERSION,
+            endpoint_override=zun_endpoint.url,
+            auth_url=keystone_endpoint.url,
+            session=session_and_auth['auth']
+        )
+
+        discovered_version = zun_api_versions.discover_version(
+            temp_client,
+            zun_api_versions.APIVersion(zun_api_versions.MAX_API_VERSION)
+        )
+
+        return self._get_client_class()(
+            discovered_version,
             endpoint_override=zun_endpoint.url,
             auth_url=keystone_endpoint.url,
             session=session_and_auth['session']
         )
 
-        return client
-
     @classmethod
     def _get_fake_client(cls):
         session = keystone_utils.get_admin_session()
