@@ -42,13 +42,24 @@ class EchoAction(actions.Action):
     stub.
     """
 
-    def __init__(self, output):
+    def __init__(self, output, delay=0):
         super(EchoAction, self).__init__()
 
         self.output = output
+        try:
+            self._delay = float(delay)
+            self._delay = 0 if self._delay < 0 else self._delay
+        except ValueError:
+            self._delay = 0
 
     def run(self, context):
-        LOG.info('Running echo action [output=%s]', self.output)
+        LOG.info(
+            'Running echo action [output=%s, delay=%s]',
+            self.output,
+            self._delay
+        )
+
+        time.sleep(self._delay)
 
         return self.output
 
@@ -278,9 +289,9 @@ class MistralHTTPAction(HTTPAction):
 
 
 class SendEmailAction(actions.Action):
-    def __init__(self, from_addr, to_addrs, smtp_server, cc_addrs=None,
-                 bcc_addrs=None, smtp_password=None, subject=None, body=None,
-                 html_body=None):
+    def __init__(self, from_addr, to_addrs, smtp_server, reply_to=None,
+                 cc_addrs=None, bcc_addrs=None, smtp_password=None,
+                 subject=None, body=None, html_body=None):
         super(SendEmailAction, self).__init__()
         # TODO(dzimine): validate parameters
 
@@ -288,6 +299,7 @@ class SendEmailAction(actions.Action):
         self.to = to_addrs
         self.cc = cc_addrs or []
         self.bcc = bcc_addrs or []
+        self.reply_to = reply_to or []
         self.subject = subject or "<No subject>"
         self.body = body or "<No body>"
         self.html_body = html_body
@@ -300,10 +312,11 @@ class SendEmailAction(actions.Action):
     def run(self, context):
         LOG.info(
             "Sending email message "
-            "[from=%s, to=%s, cc=%s, bcc=%s, subject=%s, using smtp=%s, "
-            "body=%s...]",
+            "[from=%s, to=%s, reply_to=%s, cc=%s, bcc=%s, subject=%s, "
+            "using smtp=%s, body=%s...]",
             self.sender,
             self.to,
+            self.reply_to,
             self.cc,
             self.bcc,
             self.subject,
@@ -322,6 +335,7 @@ class SendEmailAction(actions.Action):
                                          _charset='utf-8'))
         message['Subject'] = header.Header(self.subject, 'utf-8')
         message['From'] = self.sender
+        message['Reply-To'] = header.Header(', '.join(self.reply_to))
         message['To'] = ', '.join(self.to)
 
         if self.cc:
@@ -351,10 +365,11 @@ class SendEmailAction(actions.Action):
         # to return a result.
         LOG.info(
             "Sending email message "
-            "[from=%s, to=%s, cc=%s, bcc=%s, subject=%s, using smtp=%s, "
-            "body=%s...]",
+            "[from=%s, to=%s, reply_to=%s, cc=%s, bcc=%s, subject=%s, "
+            " using smtp=%s, body=%s...]",
             self.sender,
             self.to,
+            self.reply_to,
             self.cc,
             self.bcc,
             self.subject,
@@ -376,7 +391,7 @@ class SSHAction(actions.Action):
         return ssh_utils.execute_command
 
     def __init__(self, cmd, host, username,
-                 password="", private_key_filename=None):
+                 password="", private_key_filename=None, private_key=None):
         super(SSHAction, self).__init__()
 
         self.cmd = cmd
@@ -384,13 +399,15 @@ class SSHAction(actions.Action):
         self.username = username
         self.password = password
         self.private_key_filename = private_key_filename
+        self.private_key = private_key
 
         self.params = {
             'cmd': self.cmd,
             'host': self.host,
             'username': self.username,
             'password': self.password,
-            'private_key_filename': self.private_key_filename
+            'private_key_filename': self.private_key_filename,
+            'private_key': self.private_key
         }
 
     def run(self, context):
@@ -438,13 +455,15 @@ class SSHProxiedAction(SSHAction):
 
     def __init__(self, cmd, host, username, private_key_filename,
                  gateway_host, gateway_username=None,
-                 password=None, proxy_command=None):
+                 password=None, proxy_command=None,
+                 private_key=None):
         super(SSHProxiedAction, self).__init__(
             cmd,
             host,
             username,
             password,
-            private_key_filename
+            private_key_filename,
+            private_key
         )
 
         self.gateway_host = gateway_host
