@@ -34,9 +34,10 @@ from mistral import exceptions as exc
 from mistral.rpc import clients as rpc
 from mistral.services import workflows as wf_service
 from mistral.utils import filter_utils
-from mistral.utils import merge_dicts
 from mistral.utils import rest_utils
+from mistral.workflow import data_flow
 from mistral.workflow import states
+from mistral_lib.utils import merge_dicts
 
 LOG = logging.getLogger(__name__)
 
@@ -51,17 +52,14 @@ STATE_TYPES = wtypes.Enum(
 )
 
 
-def _load_deferred_output_field(ex):
-    if ex:
-        # We need to refer to this lazy-load field explicitly in
-        # order to make sure that it is correctly loaded.
-        hasattr(ex, 'output')
+def _get_workflow_execution_resource_with_output(wf_ex):
+    rest_utils.load_deferred_fields(wf_ex, ['params', 'input', 'output'])
 
-    return ex
+    return resources.Execution.from_db_model(wf_ex)
 
 
 def _get_workflow_execution_resource(wf_ex):
-    _load_deferred_output_field(wf_ex)
+    rest_utils.load_deferred_fields(wf_ex, ['params', 'input'])
 
     return resources.Execution.from_db_model(wf_ex)
 
@@ -75,7 +73,10 @@ def _get_workflow_execution(id, must_exist=True):
         else:
             wf_ex = db_api.load_workflow_execution(id)
 
-        return _load_deferred_output_field(wf_ex)
+        return rest_utils.load_deferred_fields(
+            wf_ex,
+            ['params', 'input', 'output', 'context', 'spec']
+        )
 
 
 # TODO(rakhmerov): Make sure to make all needed renaming on public API.
@@ -98,7 +99,13 @@ class ExecutionsController(rest.RestController):
 
         wf_ex = _get_workflow_execution(id)
 
-        return resources.Execution.from_db_model(wf_ex)
+        resource = resources.Execution.from_db_model(wf_ex)
+
+        resource.published_global = (
+            data_flow.get_workflow_execution_published_global(wf_ex)
+        )
+
+        return resource
 
     @rest_utils.wrap_wsme_controller_exception
     @wsme_pecan.wsexpose(
@@ -232,14 +239,18 @@ class ExecutionsController(rest.RestController):
 
         if not exec_id:
             exec_id = uuidutils.generate_uuid()
+
             LOG.debug("Generated execution id [exec_id=%s]", exec_id)
+
             exec_dict.update({'id': exec_id})
+
             wf_ex = None
         else:
             # If ID is present we need to check if such execution exists.
             # If yes, the method just returns the object. If not, the ID
             # will be used to create a new execution.
             wf_ex = _get_workflow_execution(exec_id, must_exist=False)
+
             if wf_ex:
                 return resources.Execution.from_db_model(wf_ex)
 
@@ -255,7 +266,9 @@ class ExecutionsController(rest.RestController):
                 source_execution_id).to_dict()
 
             exec_dict['description'] = "{} Based on the execution '{}'".format(
-                exec_dict['description'], source_execution_id)
+                exec_dict['description'],
+                source_execution_id
+            )
             exec_dict['description'] = exec_dict['description'].strip()
 
         result_exec_dict = merge_dicts(source_exec_dict, exec_dict)
@@ -270,8 +283,10 @@ class ExecutionsController(rest.RestController):
         engine = rpc.get_engine_client()
 
         result = engine.start_workflow(
-            result_exec_dict.get('workflow_id',
-                                 result_exec_dict.get('workflow_name')),
+            result_exec_dict.get(
+                'workflow_id',
+                result_exec_dict.get('workflow_name')
+            ),
             result_exec_dict.get('workflow_namespace', ''),
             result_exec_dict.get('id'),
             result_exec_dict.get('input'),
@@ -398,9 +413,9 @@ class ExecutionsController(rest.RestController):
         )
 
         if include_output:
-            resource_function = _get_workflow_execution_resource
+            resource_function = _get_workflow_execution_resource_with_output
         else:
-            resource_function = None
+            resource_function = _get_workflow_execution_resource
 
         return rest_utils.get_all(
             resources.Executions,
