@@ -19,8 +19,8 @@ from wsme import types as wtypes
 from mistral.api.controllers import resource
 from mistral.api.controllers.v2 import types
 from mistral import exceptions as exc
-from mistral import utils
 from mistral.workflow import states
+from mistral_lib import utils
 
 SCOPE_TYPES = wtypes.Enum(str, 'private', 'public')
 
@@ -314,6 +314,8 @@ class Execution(resource.Resource):
 
     project_id = wsme.wsattr(wtypes.text, readonly=True)
 
+    published_global = types.jsontype
+
     @classmethod
     def sample(cls):
         return cls(
@@ -326,6 +328,7 @@ class Execution(resource.Resource):
             state='SUCCESS',
             input={},
             output={},
+            published_global={'key': 'value'},
             params={
                 'env': {'k1': 'abc', 'k2': 123},
                 'notify': [
@@ -399,6 +402,7 @@ class Task(resource.Resource):
 
     result = wtypes.text
     published = types.jsontype
+    published_global = types.jsontype
     processed = bool
 
     created_at = wtypes.text
@@ -432,6 +436,7 @@ class Task(resource.Resource):
             },
             result='task result',
             published={'key': 'value'},
+            published_global={'key': 'value'},
             processed=True,
             created_at='1970-01-01T00:00:00.000000',
             updated_at='1970-01-01T00:00:00.000000',
@@ -784,6 +789,7 @@ class TaskExecutionReportEntry(BaseExecutionReportEntry):
 
     action_executions = [ActionExecutionReportEntry]
     workflow_executions = [WorkflowExecutionReportEntry]
+    retry_count = wtypes.IntegerType(minimum=0)
 
     @classmethod
     def sample(cls):
@@ -791,6 +797,7 @@ class TaskExecutionReportEntry(BaseExecutionReportEntry):
 
         sample.action_executions = [ActionExecutionReportEntry.sample()]
         sample.workflow_executions = []
+        sample.retry_count = 0
 
         return sample
 
