@@ -27,11 +27,11 @@ from pecan import hooks
 
 from mistral import auth
 from mistral import exceptions as exc
-from mistral import utils
+from mistral_lib import utils
 
 CONF = cfg.CONF
 _CTX_THREAD_LOCAL_NAME = "MISTRAL_APP_CTX_THREAD_LOCAL"
-ALLOWED_WITHOUT_AUTH = ['/', '/v2/']
+ALLOWED_WITHOUT_AUTH = ['/', '/v2/', '/workflowv2/', '/workflowv2/v2/']
 
 
 class MistralContext(oslo_context.RequestContext):
