@@ -13,8 +13,11 @@
 #    See the License for the specific language governing permissions and
 #    limitations under the License.
 
+import cachetools
 import contextlib
+import threading
 
+from oslo_config import cfg
 from oslo_db import api as db_api
 
 
@@ -24,6 +27,15 @@ _BACKEND_MAPPING = {
 
 IMPL = db_api.DBAPI('sqlalchemy', backend_mapping=_BACKEND_MAPPING)
 
+CONF = cfg.CONF
+
+_ACTION_DEF_CACHE = cachetools.TTLCache(
+    maxsize=1000,
+    ttl=CONF.engine.action_definition_cache_time  # 60 seconds by default
+)
+
+_ACTION_DEF_CACHE_LOCK = threading.RLock()
+
 
 def setup_db():
     IMPL.setup_db()
@@ -179,7 +191,20 @@ def get_action_definition(name, fields=()):
 
 def load_action_definition(name, fields=()):
     """Unlike get_action_definition this method is allowed to return None."""
-    return IMPL.load_action_definition(name, fields=fields)
+    with _ACTION_DEF_CACHE_LOCK:
+        action_def = _ACTION_DEF_CACHE.get(name)
+
+    if action_def:
+        return action_def
+
+    action_def = IMPL.load_action_definition(name, fields=fields)
+
+    with _ACTION_DEF_CACHE_LOCK:
+        _ACTION_DEF_CACHE[name] = (
+            action_def.get_clone() if action_def else None
+        )
+
+    return action_def
 
 
 def get_action_definitions(limit=None, marker=None, sort_keys=None,
@@ -240,6 +265,10 @@ def create_or_update_action_execution(id, values):
     return IMPL.create_or_update_action_execution(id, values)
 
 
+def update_action_execution_heartbeat(id):
+    return IMPL.update_action_execution_heartbeat(id)
+
+
 def delete_action_execution(id):
     return IMPL.delete_action_execution(id)
 
@@ -316,6 +345,10 @@ def get_task_executions(limit=None, marker=None, sort_keys=None,
     )
 
 
+def get_task_executions_count(**kwargs):
+    return IMPL.get_task_executions_count(**kwargs)
+
+
 def get_completed_task_executions(**kwargs):
     return IMPL.get_completed_task_executions(**kwargs)
 
@@ -420,6 +453,10 @@ def delete_scheduled_jobs(**kwargs):
     return IMPL.delete_scheduled_jobs(**kwargs)
 
 
+def get_scheduled_jobs_count(**kwargs):
+    return IMPL.get_scheduled_jobs_count(**kwargs)
+
+
 # Cron triggers.
 
 def get_cron_trigger(identifier):
