@@ -15,9 +15,12 @@
 #    limitations under the License.
 
 import contextlib
+import datetime
+import re
 import sys
 import threading
 
+
 from oslo_config import cfg
 from oslo_db import exception as db_exc
 from oslo_db import sqlalchemy as oslo_sqlalchemy
@@ -35,8 +38,8 @@ from mistral.db.v2.sqlalchemy import filters as db_filters
 from mistral.db.v2.sqlalchemy import models
 from mistral import exceptions as exc
 from mistral.services import security
-from mistral import utils
 from mistral.workflow import states
+from mistral_lib import utils
 
 
 CONF = cfg.CONF
@@ -256,7 +259,9 @@ def _delete_all(model, **kwargs):
     # NOTE(kong): Because we use 'in_' operator in _secure_query(), delete()
     # method will raise error with default parameter. Please refer to
     # http://docs.sqlalchemy.org/en/rel_1_0/orm/query.html#sqlalchemy.orm.query.Query.delete
-    _secure_query(model).filter_by(**kwargs).delete(synchronize_session=False)
+    query = _secure_query(model)
+    query = db_filters.apply_filters(query, model, **kwargs)
+    query.delete(synchronize_session=False)
 
 
 def _get_collection(model, insecure=False, limit=None, marker=None,
@@ -782,6 +787,17 @@ def create_or_update_action_execution(id, values, session=None):
         return update_action_execution(id, values)
 
 
+@b.session_aware()
+def update_action_execution_heartbeat(id, session=None):
+    if not id:
+        raise exc.DBEntityNotFoundError
+
+    now = utils.utc_now_sec()
+    session.query(models.ActionExecution).\
+        filter(models.ActionExecution.id == id).\
+        update({'last_heartbeat': now})
+
+
 @b.session_aware()
 def delete_action_execution(id, session=None):
     count = _secure_query(models.ActionExecution).filter(
@@ -876,13 +892,59 @@ def delete_workflow_execution(id, session=None):
     insecure = context.ctx().is_admin
     query = b.model_query(model) if insecure else _secure_query(model)
 
-    count = query.filter(
-        models.WorkflowExecution.id == id).delete()
+    try:
+        count = query.filter(
+            models.WorkflowExecution.id == id
+        ).delete()
 
-    if count == 0:
-        raise exc.DBEntityNotFoundError(
-            "WorkflowExecution not found [id=%s]" % id
-        )
+        if count == 0:
+            raise exc.DBEntityNotFoundError(
+                "WorkflowExecution not found [id=%s]" % id
+            )
+    except db_exc.DBError as e:
+        if is_mysql_max_depth_error(e) or is_mariadb_max_depth_error(e):
+            # https://bugs.launchpad.net/mistral/+bug/1832300
+            # mysql cascade delete error
+            delete_workflow_execution_recurse(id)
+        else:
+            raise
+
+
+def is_mysql_max_depth_error(e):
+    pattern = ".*3008.*Foreign key cascade delete" \
+              "/update exceeds max depth of 15.*"
+    return re.match(pattern, str(e))
+
+
+def is_mariadb_max_depth_error(e):
+    pattern = ".*Got error 193.*ON DELETE CASCADE.*"
+    return re.match(pattern, str(e))
+
+
+def delete_workflow_execution_recurse(wf_ex_id):
+    sub_wf_ex_ids = _get_all_direct_subworkflows(wf_ex_id)
+
+    for sub_wf_ex_id in sub_wf_ex_ids:
+        delete_workflow_execution(sub_wf_ex_id)
+
+    delete_workflow_execution(wf_ex_id)
+
+
+def _get_all_direct_subworkflows(wf_ex_id):
+    model = models.WorkflowExecution
+    insecure = context.ctx().is_admin
+    if insecure:
+        query = b.model_query(model, [model.id])
+    else:
+        query = _secure_query(model, model.id)
+    query = query.join(
+        models.TaskExecution,
+        models.WorkflowExecution.task_execution_id == models.TaskExecution.id
+    ).filter(
+        models.TaskExecution.workflow_execution_id == wf_ex_id
+    )
+
+    return [i[0] for i in query.all()]
 
 
 @b.session_aware()
@@ -920,6 +982,15 @@ def get_task_executions(session=None, **kwargs):
     return _get_collection(models.TaskExecution, **kwargs)
 
 
+@b.session_aware()
+def get_task_executions_count(session=None, **kwargs):
+    query = b.model_query(models.TaskExecution)
+
+    query = query.filter_by(**kwargs)
+
+    return query.count()
+
+
 def _get_completed_task_executions_query(kwargs):
     query = b.model_query(models.TaskExecution)
 
@@ -1150,6 +1221,130 @@ def delete_delayed_calls(session=None, **kwargs):
     return _delete_all(models.DelayedCall, **kwargs)
 
 
+# Scheduled jobs.
+
+@b.session_aware()
+def create_scheduled_job(values, session=None):
+    job = models.ScheduledJob()
+
+    job.update(values.copy())
+
+    try:
+        job.save(session)
+    except db_exc.DBDuplicateEntry as e:
+        raise exc.DBDuplicateEntryError(
+            "Duplicate entry for ScheduledJob ID: {}".format(e.value)
+        )
+
+    return job
+
+
+@b.session_aware()
+def get_scheduled_jobs_to_start(time, batch_size=None, session=None):
+    query = b.model_query(models.ScheduledJob)
+
+    execute_at_col = models.ScheduledJob.execute_at
+    captured_at_col = models.ScheduledJob.captured_at
+
+    # Filter by execution time accounting for a configured job pickup interval.
+    # TODO(rakhmerov): Configuration options should not be accessed here.
+    query = query.filter(
+        execute_at_col <
+        time - datetime.timedelta(seconds=CONF.scheduler.pickup_job_after)
+    )
+
+    # Filter by captured time accounting for a configured captured job timeout.
+    min_captured_at = (
+        utils.utc_now_sec() -
+        datetime.timedelta(seconds=CONF.scheduler.captured_job_timeout)
+    )
+
+    query = query.filter(
+        sa.or_(
+            captured_at_col == sa.null(),
+            captured_at_col <= min_captured_at
+        )
+    )
+
+    query = query.order_by(execute_at_col)
+    query = query.limit(batch_size)
+
+    return query.all()
+
+
+@b.session_aware()
+def update_scheduled_job(id, values, query_filter=None, session=None):
+    if query_filter:
+        try:
+            specimen = models.ScheduledJob(id=id, **query_filter)
+
+            job = b.model_query(
+                models.ScheduledJob
+            ).update_on_match(
+                specimen=specimen,
+                surrogate_key='id',
+                values=values
+            )
+
+            return job, 1
+
+        except oslo_sqlalchemy.update_match.NoRowsMatched as e:
+            LOG.debug(
+                "No rows matched for update scheduled job [id=%s, values=%s, "
+                "query_filter=%s,"
+                "exception=%s]", id, values, query_filter, e
+            )
+
+            return None, 0
+
+    else:
+        job = get_scheduled_job(id=id, session=session)
+
+        job.update(values)
+
+        return job, len(session.dirty)
+
+
+@b.session_aware()
+def get_scheduled_job(id, session=None):
+    job = _get_db_object_by_id(models.ScheduledJob, id)
+
+    if not job:
+        raise exc.DBEntityNotFoundError(
+            "Scheduled job not found [id=%s]" % id
+        )
+
+    return job
+
+
+@b.session_aware()
+def delete_scheduled_job(id, session=None):
+    # It's safe to use insecure query here because users can't access
+    # scheduled job.
+    count = b.model_query(models.ScheduledJob).filter(
+        models.ScheduledJob.id == id).delete()
+
+    if count == 0:
+        raise exc.DBEntityNotFoundError(
+            "Scheduled job not found [id=%s]" % id
+        )
+
+
+def get_scheduled_jobs(**kwargs):
+    return _get_collection(model=models.ScheduledJob, **kwargs)
+
+
+@b.session_aware()
+def delete_scheduled_jobs(session=None, **kwargs):
+    return _delete_all(models.ScheduledJob, **kwargs)
+
+
+def get_scheduled_jobs_count(**kwargs):
+    return _get_count(model=models.ScheduledJob, **kwargs)
+
+
+# Other functions.
+
 @b.session_aware()
 def get_expired_executions(expiration_time, limit=None, columns=(),
                            session=None):
