@@ -29,7 +29,7 @@ from mistral.db.sqlalchemy import model_base as mb
 from mistral.db.sqlalchemy import types as st
 from mistral import exceptions as exc
 from mistral.services import security
-from mistral import utils
+from mistral_lib import utils
 
 
 # Definition objects.
@@ -177,7 +177,6 @@ class Execution(mb.MistralSecureModelBase):
     workflow_name = sa.Column(sa.String(255))
     workflow_namespace = sa.Column(sa.String(255))
     workflow_id = sa.Column(sa.String(80))
-    spec = sa.Column(st.JsonMediumDictType())
     state = sa.Column(sa.String(20))
     state_info = sa.Column(sa.Text(), nullable=True)
     tags = sa.Column(st.JsonListType())
@@ -199,6 +198,7 @@ class ActionExecution(Execution):
     )
 
     # Main properties.
+    spec = sa.Column(st.JsonMediumDictType())
     accepted = sa.Column(sa.Boolean(), default=False)
     input = sa.Column(st.JsonLongDictType(), nullable=True)
     output = sa.orm.deferred(sa.Column(st.JsonLongDictType(), nullable=True))
@@ -224,10 +224,11 @@ class WorkflowExecution(Execution):
     )
 
     # Main properties.
+    spec = sa.orm.deferred(sa.Column(st.JsonMediumDictType()))
     accepted = sa.Column(sa.Boolean(), default=False)
-    input = sa.Column(st.JsonLongDictType(), nullable=True)
+    input = sa.orm.deferred(sa.Column(st.JsonLongDictType(), nullable=True))
     output = sa.orm.deferred(sa.Column(st.JsonLongDictType(), nullable=True))
-    params = sa.Column(st.JsonLongDictType())
+    params = sa.orm.deferred(sa.Column(st.JsonLongDictType()))
 
     # Initial workflow context containing workflow variables, environment,
     # openstack security context etc.
@@ -235,7 +236,7 @@ class WorkflowExecution(Execution):
     #   * Data stored in this structure should not be copied into inbound
     #     contexts of tasks. No need to duplicate it.
     #   * This structure does not contain workflow input.
-    context = sa.Column(st.JsonLongDictType())
+    context = sa.orm.deferred(sa.Column(st.JsonLongDictType()))
 
 
 class TaskExecution(Execution):
@@ -252,6 +253,7 @@ class TaskExecution(Execution):
     )
 
     # Main properties.
+    spec = sa.orm.deferred(sa.Column(st.JsonMediumDictType()))
     action_spec = sa.Column(st.JsonLongDictType())
     unique_key = sa.Column(sa.String(255), nullable=True)
     type = sa.Column(sa.String(10))
@@ -263,6 +265,23 @@ class TaskExecution(Execution):
     # significantly.
     processed = sa.Column(sa.BOOLEAN, default=False)
 
+    # Set to True if the completion of the task led to starting new
+    # tasks.
+    # The value of this property should be ignored if the task
+    # is not completed.
+    has_next_tasks = sa.Column(sa.Boolean, default=False)
+
+    # The names of the next tasks.
+    # [(task_name, event)]
+    next_tasks = sa.Column(st.JsonListType())
+
+    # Set to True if the task finished with an error and the error
+    # is handled (e.g. with 'on-error' clause for direct workflows)
+    # so that the error shouldn't bubble up to the workflow level.
+    # The value of this property should be ignored if the task
+    # is not completed.
+    error_handled = sa.Column(sa.Boolean, default=False)
+
     # Data Flow properties.
     in_context = sa.Column(st.JsonLongDictType())
     published = sa.Column(st.JsonLongDictType())
@@ -438,11 +457,7 @@ class ScheduledJob(mb.MistralModelBase):
     execute_at = sa.Column(sa.DateTime, nullable=False)
     captured_at = sa.Column(sa.DateTime, nullable=True)
 
-
-sa.Index(
-    '%s_execution_time' % ScheduledJob.__tablename__,
-    ScheduledJob.execute_at
-)
+    key = sa.Column(sa.String(250), nullable=True)
 
 
 class Environment(mb.MistralSecureModelBase):
