@@ -28,17 +28,18 @@ from mistral import exceptions as exc
 from mistral.executors import base as exe
 from mistral import expressions as expr
 from mistral.lang import parser as spec_parser
+from mistral.rpc import clients as rpc
 from mistral.services import action_manager as a_m
 from mistral.services import security
-from mistral import utils
 from mistral.utils import wf_trace
 from mistral.workflow import data_flow
-from mistral.workflow import lookup_utils
 from mistral.workflow import states
 from mistral_lib import actions as ml_actions
+from mistral_lib import utils
 
 
 LOG = logging.getLogger(__name__)
+CONF = cfg.CONF
 
 
 @six.add_metaclass(abc.ABCMeta)
@@ -322,11 +323,16 @@ class PythonAction(Action):
         return self._prepare_output(result)
 
     def is_sync(self, input_dict):
-        prepared_input_dict = self._prepare_input(input_dict)
+        try:
+            prepared_input_dict = self._prepare_input(input_dict)
 
-        a = a_m.get_action_class(self.action_def.name)(**prepared_input_dict)
+            a = a_m.get_action_class(self.action_def.name)(
+                **prepared_input_dict
+            )
 
-        return a.is_sync()
+            return a.is_sync()
+        except BaseException as e:
+            raise exc.InputException(str(e))
 
     def validate_input(self, input_dict):
         # NOTE(kong): Don't validate action input if action initialization
@@ -392,7 +398,7 @@ class AdHocAction(PythonAction):
                  wf_ctx=None):
         self.action_spec = spec_parser.get_action_spec(action_def.spec)
 
-        base_action_def = lookup_utils.find_action_definition_by_name(
+        base_action_def = db_api.load_action_definition(
             self.action_spec.get_base()
         )
 
@@ -611,14 +617,28 @@ class WorkflowAction(Action):
                 wf_params[k] = v
                 del input_dict[k]
 
-        wf_handler.start_workflow(
-            wf_def.id,
-            wf_def.namespace,
-            None,
-            input_dict,
-            "sub-workflow execution",
-            wf_params
-        )
+        if cfg.CONF.engine.start_subworkflows_via_rpc:
+            def _start_subworkflow():
+                rpc.get_engine_client().start_workflow(
+                    wf_def.id,
+                    wf_def.namespace,
+                    None,
+                    input_dict,
+                    "sub-workflow execution",
+                    async_=True,
+                    **wf_params
+                )
+
+            post_tx_queue.register_operation(_start_subworkflow)
+        else:
+            wf_handler.start_workflow(
+                wf_def.id,
+                wf_def.namespace,
+                None,
+                input_dict,
+                "sub-workflow execution",
+                wf_params
+            )
 
     @profiler.trace('workflow-action-run', hide_args=True)
     def run(self, input_dict, target, index=0, desc='', save=True,
@@ -656,14 +676,10 @@ def resolve_action_definition(action_spec_name, wf_name=None,
 
         action_full_name = "%s.%s" % (wb_name, action_spec_name)
 
-        action_db = lookup_utils.find_action_definition_by_name(
-            action_full_name
-        )
+        action_db = db_api.load_action_definition(action_full_name)
 
     if not action_db:
-        action_db = lookup_utils.find_action_definition_by_name(
-            action_spec_name
-        )
+        action_db = db_api.load_action_definition(action_spec_name)
 
     if not action_db:
         raise exc.InvalidActionException(
