@@ -29,7 +29,7 @@ class Engine(object):
 
     @abc.abstractmethod
     def start_workflow(self, wf_identifier, wf_namespace='', wf_ex_id=None,
-                       wf_input=None, description='', **params):
+                       wf_input=None, description='', async_=False, **params):
         """Starts the specified workflow.
 
         :param wf_identifier: Workflow ID or name. Workflow ID is recommended,
@@ -39,6 +39,8 @@ class Engine(object):
         :param wf_ex_id: Workflow execution id. If passed, it will be set
             in the new execution object.
         :param description: Execution description.
+        :param async_: If True, start workflow in asynchronous mode
+            (w/o waiting for completion).
         :param params: Additional workflow type specific parameters.
         :return: Workflow execution object.
         """
@@ -131,6 +133,14 @@ class Engine(object):
         """
         raise NotImplementedError
 
+    @abc.abstractmethod
+    def report_running_actions(self, action_ex_ids):
+        """Receives the heartbeat about the running actions.
+
+        :param action_ex_ids: The action execution ids.
+        """
+        raise NotImplementedError
+
 
 @six.add_metaclass(abc.ABCMeta)
 class TaskPolicy(object):
@@ -152,6 +162,8 @@ class TaskPolicy(object):
 
         ctx_view = data_flow.ContextView(
             task_ex.in_context,
+            data_flow.get_current_task_dict(task_ex),
+            data_flow.get_workflow_environment_dict(wf_ex),
             wf_ex.context,
             wf_ex.input
         )
@@ -170,6 +182,8 @@ class TaskPolicy(object):
 
         ctx_view = data_flow.ContextView(
             task_ex.in_context,
+            data_flow.get_current_task_dict(task_ex),
+            data_flow.get_workflow_environment_dict(wf_ex),
             wf_ex.context,
             wf_ex.input
         )
