@@ -19,7 +19,7 @@ from mistral.engine import base
 from mistral.engine import post_tx_queue
 from mistral.engine import workflow_handler as wf_handler
 from mistral import expressions
-from mistral.services import scheduler
+from mistral.scheduler import base as sched_base
 from mistral.utils import wf_trace
 from mistral.workflow import data_flow
 from mistral.workflow import states
@@ -33,11 +33,11 @@ _FAIL_IF_INCOMPLETE_TASK_PATH = (
 )
 
 
-def _log_task_delay(task_ex, delay_sec):
+def _log_task_delay(task_ex, delay_sec, state=states.RUNNING_DELAYED):
     wf_trace.info(
         task_ex,
         "Task '%s' [%s -> %s, delay = %s sec]" %
-        (task_ex.name, task_ex.state, states.RUNNING_DELAYED, delay_sec)
+        (task_ex.name, task_ex.state, state, delay_sec)
     )
 
 
@@ -56,6 +56,7 @@ def get_policy_factories():
         build_pause_before_policy,
         build_wait_before_policy,
         build_wait_after_policy,
+        build_fail_on_policy,
         build_retry_policy,
         build_timeout_policy,
         build_concurrency_policy
@@ -150,6 +151,16 @@ def build_concurrency_policy(policies_spec):
             if concurrency_policy else None)
 
 
+def build_fail_on_policy(policies_spec):
+    if not policies_spec:
+        return None
+
+    fail_on_policy = policies_spec.get_fail_on()
+
+    return (FailOnPolicy(fail_on_policy)
+            if fail_on_policy else None)
+
+
 def _ensure_context_has_key(runtime_context, key):
     if not runtime_context:
         runtime_context = {}
@@ -160,15 +171,6 @@ def _ensure_context_has_key(runtime_context, key):
     return runtime_context
 
 
-def _has_incomplete_inbound_tasks(task_ex):
-    if "triggered_by" not in task_ex.runtime_context:
-        return False
-    for trigger in task_ex.runtime_context["triggered_by"]:
-        if trigger["event"] == "not triggered":
-            return True
-    return False
-
-
 class WaitBeforePolicy(base.TaskPolicy):
     _schema = {
         "properties": {
@@ -219,13 +221,18 @@ class WaitBeforePolicy(base.TaskPolicy):
 
             task_ex.state = states.RUNNING_DELAYED
 
-            scheduler.schedule_call(
-                None,
-                _CONTINUE_TASK_PATH,
-                self.delay,
-                task_ex_id=task_ex.id,
+            sched = sched_base.get_system_scheduler()
+
+            job = sched_base.SchedulerJob(
+                run_after=self.delay,
+                func_name=_CONTINUE_TASK_PATH,
+                func_args={
+                    'task_ex_id': task_ex.id
+                }
             )
 
+            sched.schedule(job)
+
 
 class WaitAfterPolicy(base.TaskPolicy):
     _schema = {
@@ -278,15 +285,20 @@ class WaitAfterPolicy(base.TaskPolicy):
         )
 
         # Schedule to change task state to RUNNING again.
-        scheduler.schedule_call(
-            None,
-            _COMPLETE_TASK_PATH,
-            self.delay,
-            task_ex_id=task_ex.id,
-            state=end_state,
-            state_info=end_state_info
+        sched = sched_base.get_system_scheduler()
+
+        job = sched_base.SchedulerJob(
+            run_after=self.delay,
+            func_name=_COMPLETE_TASK_PATH,
+            func_args={
+                'task_ex_id': task_ex.id,
+                'state': end_state,
+                'state_info': end_state_info
+            }
         )
 
+        sched.schedule(job)
+
 
 class RetryPolicy(base.TaskPolicy):
     _schema = {
@@ -390,11 +402,6 @@ class RetryPolicy(base.TaskPolicy):
             (self._continue_on_clause and not continue_on_evaluation)
         )
 
-        stop_continue_flag = (
-            stop_continue_flag or
-            _has_incomplete_inbound_tasks(task_ex)
-        )
-
         break_triggered = (
             task_ex.state == states.ERROR and
             break_on_evaluation
@@ -403,22 +410,39 @@ class RetryPolicy(base.TaskPolicy):
         if not retries_remain or break_triggered or stop_continue_flag:
             return
 
-        _log_task_delay(task_ex, self.delay)
-
         data_flow.invalidate_task_execution_result(task_ex)
 
-        task_ex.state = states.RUNNING_DELAYED
-
         policy_context['retry_no'] = retry_no + 1
         runtime_context[context_key] = policy_context
 
-        scheduler.schedule_call(
-            None,
-            _CONTINUE_TASK_PATH,
-            self.delay,
-            task_ex_id=task_ex.id,
+        # NOTE(vgvoleg): join tasks in direct workflows can't be
+        # retried as is, because this tasks can't start without
+        # the correct logical state.
+        if hasattr(task_spec, "get_join") and task_spec.get_join():
+            from mistral.engine import task_handler as t_h
+
+            _log_task_delay(task_ex, self.delay, states.WAITING)
+
+            task_ex.state = states.WAITING
+
+            t_h._schedule_refresh_task_state(task_ex.id, self.delay)
+
+            return
+
+        _log_task_delay(task_ex, self.delay)
+
+        task_ex.state = states.RUNNING_DELAYED
+
+        sched = sched_base.get_system_scheduler()
+
+        job = sched_base.SchedulerJob(
+            run_after=self.delay,
+            func_name=_CONTINUE_TASK_PATH,
+            func_args={'task_ex_id': task_ex.id}
         )
 
+        sched.schedule(job)
+
     @staticmethod
     def refresh_runtime_context(task_ex):
         runtime_context = task_ex.runtime_context or {}
@@ -449,14 +473,19 @@ class TimeoutPolicy(base.TaskPolicy):
         if self.delay == 0:
             return
 
-        scheduler.schedule_call(
-            None,
-            _FAIL_IF_INCOMPLETE_TASK_PATH,
-            self.delay,
-            task_ex_id=task_ex.id,
-            timeout=self.delay
+        sched = sched_base.get_system_scheduler()
+
+        job = sched_base.SchedulerJob(
+            run_after=self.delay,
+            func_name=_FAIL_IF_INCOMPLETE_TASK_PATH,
+            func_args={
+                'task_ex_id': task_ex.id,
+                'timeout': self.delay
+            }
         )
 
+        sched.schedule(job)
+
         wf_trace.info(
             task_ex,
             "Timeout check scheduled [task=%s, timeout(s)=%s]." %
@@ -524,13 +553,39 @@ class ConcurrencyPolicy(base.TaskPolicy):
         task_ex.runtime_context = runtime_context
 
 
+class FailOnPolicy(base.TaskPolicy):
+    _schema = {
+        "properties": {
+            "fail-on": {"type": "boolean"},
+        }
+    }
+
+    def __init__(self, fail_on):
+        self.fail_on = fail_on
+
+    def before_task_start(self, task_ex, task_spec):
+        pass
+
+    def after_task_complete(self, task_ex, task_spec):
+        if task_ex.state != states.SUCCESS:
+            return
+
+        super(FailOnPolicy, self).after_task_complete(task_ex, task_spec)
+
+        if self.fail_on:
+            task_ex.state = states.ERROR
+            task_ex.state_info = 'Failed by fail-on policy'
+
+
 @db_utils.retry_on_db_error
 @post_tx_queue.run
 def _continue_task(task_ex_id):
     from mistral.engine import task_handler
 
     with db_api.transaction():
-        task_handler.continue_task(db_api.get_task_execution(task_ex_id))
+        task_ex = db_api.load_task_execution(task_ex_id)
+
+        task_handler.continue_task(task_ex)
 
 
 @db_utils.retry_on_db_error
@@ -539,7 +594,7 @@ def _complete_task(task_ex_id, state, state_info):
     from mistral.engine import task_handler
 
     with db_api.transaction():
-        task_ex = db_api.get_task_execution(task_ex_id)
+        task_ex = db_api.load_task_execution(task_ex_id)
 
         task_handler.complete_task(task_ex, state, state_info)
 
@@ -550,7 +605,7 @@ def _fail_task_if_incomplete(task_ex_id, timeout):
     from mistral.engine import task_handler
 
     with db_api.transaction():
-        task_ex = db_api.get_task_execution(task_ex_id)
+        task_ex = db_api.load_task_execution(task_ex_id)
 
         if not states.is_completed(task_ex.state):
             msg = 'Task timed out [timeout(s)=%s].' % timeout
