@@ -32,11 +32,12 @@ from mistral import exceptions as exc
 from mistral import expressions as expr
 from mistral.notifiers import base as notif
 from mistral.notifiers import notification_events as events
-from mistral import utils
 from mistral.utils import wf_trace
 from mistral.workflow import base as wf_base
+from mistral.workflow import commands
 from mistral.workflow import data_flow
 from mistral.workflow import states
+from mistral_lib import utils
 
 
 LOG = logging.getLogger(__name__)
@@ -75,13 +76,42 @@ class Task(object):
         notifier = notif.get_notifier(cfg.CONF.notifier.type)
         event = events.identify_task_event(old_task_state, new_task_state)
 
-        notifier.notify(
-            self.task_ex.id,
-            self.task_ex.to_dict(),
-            event,
-            self.task_ex.updated_at,
-            publishers
-        )
+        filtered_publishers = []
+        for publisher in publishers:
+            if not isinstance(publisher, dict):
+                continue
+
+            target_events = publisher.get('event_types', [])
+
+            if not target_events or event in target_events:
+                filtered_publishers.append(publisher)
+
+        if not filtered_publishers:
+            return
+
+        def _convert_to_notification_data():
+            return {
+                "id": self.task_ex.id,
+                "name": self.task_ex.name,
+                "workflow_execution_id": self.task_ex.workflow_execution_id,
+                "workflow_name": self.task_ex.workflow_name,
+                "workflow_namespace": self.task_ex.workflow_namespace,
+                "workflow_id": self.task_ex.workflow_id,
+                "state": self.task_ex.state,
+                "state_info": self.task_ex.state_info,
+                "type": self.task_ex.type,
+                "project_id": self.task_ex.project_id
+            }
+
+        def _send_notification():
+            notifier.notify(
+                self.task_ex.id,
+                _convert_to_notification_data(),
+                event,
+                self.task_ex.updated_at,
+                filtered_publishers
+            )
+        post_tx_queue.register_operation(_send_notification)
 
     def is_completed(self):
         return self.task_ex and states.is_completed(self.task_ex.state)
@@ -259,6 +289,29 @@ class Task(object):
         if self.task_ex.state == states.RUNNING_DELAYED:
             return
 
+        wf_ctrl = wf_base.get_controller(self.wf_ex, self.wf_spec)
+
+        # Calculate commands to process next.
+        cmds = wf_ctrl.continue_workflow(task_ex=self.task_ex)
+
+        # Save next task names in DB to avoid evaluating them again
+        # in the future.
+        self.task_ex.next_tasks = []
+
+        for c in cmds:
+            if commands.is_engine_command(c):
+                continue
+
+            event = c.triggered_by[0]['event'] if c.triggered_by else None
+
+            self.task_ex.next_tasks.append((c.task_spec.get_name(), event))
+
+        self.task_ex.has_next_tasks = bool(self.task_ex.next_tasks)
+
+        # Check whether the error is handled.
+        if self.task_ex.state == states.ERROR:
+            self.task_ex.error_handled = any([c.handles_error for c in cmds])
+
         # If workflow is paused we shouldn't schedule new commands
         # and mark task as processed.
         if states.is_paused(self.wf_ex.state):
@@ -267,11 +320,6 @@ class Task(object):
 
             return
 
-        wf_ctrl = wf_base.get_controller(self.wf_ex, self.wf_spec)
-
-        # Calculate commands to process next.
-        cmds = wf_ctrl.continue_workflow(task_ex=self.task_ex)
-
         # Mark task as processed after all decisions have been made
         # upon its completion.
         self.task_ex.processed = True
@@ -777,16 +825,8 @@ class WithItemsTask(RegularTask):
 
         :return: Evaluated 'with-items' expression values.
         """
-        ctx_view = data_flow.ContextView(
-            self.ctx,
-            self.wf_ex.context,
-            self.wf_ex.input
-        )
 
-        return expr.evaluate_recursively(
-            self.task_spec.get_with_items(),
-            ctx_view
-        )
+        return self._evaluate_expression(self.task_spec.get_with_items())
 
     def _validate_values(self, with_items_values):
         # Take only mapped values and check them.
