@@ -255,6 +255,7 @@ class DefaultEventEngine(base.EventEngine):
                 self.engine_client.start_workflow(
                     t['workflow_id'],
                     t['workflow_namespace'],
+                    None,
                     t['workflow_input'],
                     description=json.dumps(description),
                     **workflow_params
@@ -324,7 +325,7 @@ class DefaultEventEngine(base.EventEngine):
         self.handler_tg.add_thread(self._process_event_queue)
 
     def process_notification_event(self, notification):
-        """Callback funtion by event handler.
+        """Callback function by event handler.
 
         Just put notification into a queue.
         """
@@ -404,6 +405,4 @@ class DefaultEventEngine(base.EventEngine):
 
             return
 
-        security.delete_trust(trigger['trust_id'])
-
         self._add_event_listener(trigger['exchange'], trigger['topic'], events)
