@@ -33,9 +33,11 @@ V2_0 = '2.0'
 ALL_VERSIONS = [V2_0]
 
 
+# {workflow execution id => workflow specification}.
 _WF_EX_CACHE = cachetools.LRUCache(maxsize=100)
 _WF_EX_CACHE_LOCK = threading.RLock()
 
+# {(workflow def id, workflow def updated at) => workflow specification}.
 _WF_DEF_CACHE = cachetools.LRUCache(maxsize=100)
 _WF_DEF_CACHE_LOCK = threading.RLock()
 
@@ -78,15 +80,17 @@ def _get_spec_version(spec_dict):
 # Factory methods to get specifications either from raw YAML formatted text or
 # from dictionaries parsed from YAML formatted text.
 
-def get_workbook_spec(spec_dict):
+def get_workbook_spec(spec_dict, validate):
     if _get_spec_version(spec_dict) == V2_0:
-        return base.instantiate_spec(wb_v2.WorkbookSpec, spec_dict)
+        return base.instantiate_spec(
+            wb_v2.WorkbookSpec, spec_dict, validate
+        )
 
     return None
 
 
-def get_workbook_spec_from_yaml(text):
-    return get_workbook_spec(parse_yaml(text))
+def get_workbook_spec_from_yaml(text, validate=True):
+    return get_workbook_spec(parse_yaml(text), validate)
 
 
 def get_action_spec(spec_dict):
@@ -104,12 +108,14 @@ def get_action_spec_from_yaml(text, action_name):
     return get_action_spec(spec_dict)
 
 
-def get_action_list_spec(spec_dict):
-    return base.instantiate_spec(actions_v2.ActionListSpec, spec_dict)
+def get_action_list_spec(spec_dict, validate):
+    return base.instantiate_spec(
+        actions_v2.ActionListSpec, spec_dict, validate
+    )
 
 
-def get_action_list_spec_from_yaml(text):
-    return get_action_list_spec(parse_yaml(text))
+def get_action_list_spec_from_yaml(text, validate=True):
+    return get_action_list_spec(parse_yaml(text), validate=validate)
 
 
 def get_workflow_spec(spec_dict):
@@ -128,16 +134,20 @@ def get_workflow_spec(spec_dict):
     return None
 
 
-def get_workflow_list_spec(spec_dict):
-    return base.instantiate_spec(wf_v2.WorkflowListSpec, spec_dict)
+def get_workflow_list_spec(spec_dict, validate):
+    return base.instantiate_spec(
+        wf_v2.WorkflowListSpec,
+        spec_dict,
+        validate
+    )
 
 
 def get_workflow_spec_from_yaml(text):
     return get_workflow_spec(parse_yaml(text))
 
 
-def get_workflow_list_spec_from_yaml(text):
-    return get_workflow_list_spec(parse_yaml(text))
+def get_workflow_list_spec_from_yaml(text, validate=True):
+    return get_workflow_list_spec(parse_yaml(text), validate)
 
 
 def get_task_spec(spec_dict):
@@ -203,9 +213,12 @@ def get_workflow_spec_by_execution_id(wf_ex_id):
 
     The idea is that when a workflow execution is running we
     must be getting the same workflow specification even if
+    the workflow definition has already changed. However, note
+    that this is true only if the current engine instance didn't
+    restart during the entire workflow execution run.
 
     :param wf_ex_id: Workflow execution id.
-    :return:  Workflow specification.
+    :return: Workflow specification.
     """
     if not wf_ex_id:
         return None
@@ -230,8 +243,8 @@ def get_workflow_spec_by_definition_id(wf_def_id, wf_def_updated_at):
 
     :param wf_def_id: Workflow definition id.
     :param wf_def_updated_at: Workflow definition 'updated_at' value. It
-     serves only as part of cache key and is not explicitly used in the
-     method.
+        serves only as part of cache key and is not explicitly used in the
+        method.
     :return: Workflow specification.
     """
     if not wf_def_id:
