@@ -25,19 +25,24 @@ from mistral.lang.v2 import base
 from mistral.lang.v2 import on_clause
 from mistral.lang.v2 import policies
 from mistral.lang.v2 import publish
-from mistral import utils
+from mistral.lang.v2 import retry_policy
 from mistral.workflow import states
+from mistral_lib import utils
 
 _expr_ptrns = [expressions.patterns[name] for name in expressions.patterns]
 WITH_ITEMS_PTRN = re.compile(
-    "\s*([\w\d_\-]+)\s*in\s*(\[.+\]|%s)" % '|'.join(_expr_ptrns)
+    r"\s*([\w\d_\-]+)\s*in\s*(\[.+\]|%s)" % '|'.join(_expr_ptrns)
 )
-RESERVED_TASK_NAMES = [
-    'noop',
-    'fail',
-    'succeed',
-    'pause'
-]
+
+MAX_LENGTH_TASK_NAME = 255
+# Length of a join task name must be less than or equal to maximum
+# of task_executions unique_key and named_locks name. Their
+# maximum equals 255.
+# https://dev.mysql.com/doc/refman/5.6/en/innodb-restrictions.html
+# For example: "join-task-" + "workflow execution id" + "-" +
+# "task join name" = 255
+# "task join name" = 255 - 36 - 1 - 10 = MAX_LENGTH_TASK_NAME - 47
+MAX_LENGTH_JOIN_TASK_NAME = MAX_LENGTH_TASK_NAME - 47
 
 
 class TaskSpec(base.BaseSpec):
@@ -64,12 +69,13 @@ class TaskSpec(base.BaseSpec):
             },
             "publish": types.NONEMPTY_DICT,
             "publish-on-error": types.NONEMPTY_DICT,
-            "retry": types.ANY,
-            "wait-before": types.ANY,
-            "wait-after": types.ANY,
-            "timeout": types.ANY,
-            "pause-before": types.ANY,
-            "concurrency": types.ANY,
+            "retry": retry_policy.RetrySpec.get_schema(),
+            "wait-before": types.EXPRESSION_OR_POSITIVE_INTEGER,
+            "wait-after": types.EXPRESSION_OR_POSITIVE_INTEGER,
+            "timeout": types.EXPRESSION_OR_POSITIVE_INTEGER,
+            "pause-before": types.EXPRESSION_OR_BOOLEAN,
+            "concurrency": types.EXPRESSION_OR_POSITIVE_INTEGER,
+            "fail-on": types.EXPRESSION_OR_BOOLEAN,
             "target": types.NONEMPTY_STRING,
             "keep-result": types.EXPRESSION_OR_BOOLEAN,
             "safe-rerun": types.EXPRESSION_OR_BOOLEAN
@@ -97,8 +103,8 @@ class TaskSpec(base.BaseSpec):
         ]
     }
 
-    def __init__(self, data):
-        super(TaskSpec, self).__init__(data)
+    def __init__(self, data, validate):
+        super(TaskSpec, self).__init__(data, validate)
 
         self._name = data['name']
         self._description = data.get('description')
@@ -115,22 +121,23 @@ class TaskSpec(base.BaseSpec):
             'wait-after',
             'timeout',
             'pause-before',
-            'concurrency'
+            'concurrency',
+            'fail-on'
         )
         self._target = data.get('target')
         self._keep_result = data.get('keep-result', True)
-        self._safe_rerun = data.get('safe-rerun', False)
+        self._safe_rerun = data.get('safe-rerun')
 
         self._process_action_and_workflow()
 
     def validate_schema(self):
         super(TaskSpec, self).validate_schema()
 
+        self._validate_name()
+
         action = self._data.get('action')
         workflow = self._data.get('workflow')
 
-        self._validate_name()
-
         # Validate YAQL expressions.
         if action or workflow:
             inline_params = self._parse_cmd_and_input(action or workflow)[1]
@@ -145,11 +152,10 @@ class TaskSpec(base.BaseSpec):
     def _validate_name(self):
         task_name = self._data.get('name')
 
-        if task_name in RESERVED_TASK_NAMES:
+        if len(task_name) > MAX_LENGTH_TASK_NAME:
             raise exc.InvalidModelException(
-                "Reserved keyword '%s' not allowed as task name." %
-                task_name
-            )
+                "The length of a '{0}' task name must not exceed {1}"
+                " symbols".format(task_name, MAX_LENGTH_TASK_NAME))
 
     def _transform_with_items(self):
         raw = self._data.get('with-items', [])
@@ -233,10 +239,14 @@ class TaskSpec(base.BaseSpec):
         spec = None
 
         if state == states.SUCCESS and self._publish:
-            spec = publish.PublishSpec({'branch': self._publish})
+            spec = publish.PublishSpec(
+                {'branch': self._publish},
+                validate=self._validate
+            )
         elif state == states.ERROR and self._publish_on_error:
             spec = publish.PublishSpec(
-                {'branch': self._publish_on_error}
+                {'branch': self._publish_on_error},
+                validate=self._validate
             )
 
         return spec
@@ -265,9 +275,9 @@ class DirectWorkflowTaskSpec(TaskSpec):
                     types.POSITIVE_INTEGER
                 ]
             },
-            "on-complete": types.ANY,
-            "on-success": types.ANY,
-            "on-error": types.ANY
+            "on-complete": on_clause.OnClauseSpec.get_schema(),
+            "on-success": on_clause.OnClauseSpec.get_schema(),
+            "on-error": on_clause.OnClauseSpec.get_schema()
         }
     }
 
@@ -276,8 +286,8 @@ class DirectWorkflowTaskSpec(TaskSpec):
         _direct_workflow_schema
     )
 
-    def __init__(self, data):
-        super(DirectWorkflowTaskSpec, self).__init__(data)
+    def __init__(self, data, validate):
+        super(DirectWorkflowTaskSpec, self).__init__(data, validate)
 
         self._join = data.get('join')
 
@@ -293,6 +303,14 @@ class DirectWorkflowTaskSpec(TaskSpec):
         self._validate_transitions(self._on_success)
         self._validate_transitions(self._on_error)
 
+        if self._join:
+            join_task_name = self.get_name()
+            if len(join_task_name) > MAX_LENGTH_JOIN_TASK_NAME:
+                raise exc.InvalidModelException(
+                    "The length of a '{0}' join task name must not exceed {1} "
+                    "symbols".format(join_task_name, MAX_LENGTH_JOIN_TASK_NAME)
+                )
+
     def _validate_transitions(self, on_clause_spec):
         val = on_clause_spec.get_next() if on_clause_spec else []
 
@@ -353,8 +371,8 @@ class ReverseWorkflowTaskSpec(TaskSpec):
         _reverse_workflow_schema
     )
 
-    def __init__(self, data):
-        super(ReverseWorkflowTaskSpec, self).__init__(data)
+    def __init__(self, data, validate):
+        super(ReverseWorkflowTaskSpec, self).__init__(data, validate)
 
         self._requires = data.get('requires', [])
 
