@@ -21,8 +21,8 @@ from mistral.db import utils as db_utils
 from mistral.db.v2 import api as db_api
 from mistral.engine import action_handler
 from mistral.engine import post_tx_queue
-from mistral import utils
 from mistral_lib import actions as mistral_lib
+from mistral_lib import utils
 from oslo_config import cfg
 from oslo_log import log as logging
 
