@@ -2,6 +2,7 @@
 # Copyright 2015 - StackStorm, Inc.
 # Copyright 2015 Huawei Technologies Co., Ltd.
 # Copyright 2016 - Brocade Communications Systems, Inc.
+# Copyright 2018 - Extreme Networks, Inc.
 #
 #    Licensed under the Apache License, Version 2.0 (the "License");
 #    you may not use this file except in compliance with the License.
@@ -35,9 +36,9 @@ from mistral.rpc import base as rpc_base
 from mistral.rpc import clients as rpc_clients
 from mistral.tests.unit.api import base
 from mistral.tests.unit import base as unit_base
-from mistral import utils
 from mistral.utils import rest_utils
 from mistral.workflow import states
+from mistral_lib import utils
 
 # This line is needed for correct initialization of messaging config.
 oslo_messaging.get_rpc_transport(cfg.CONF)
@@ -51,6 +52,7 @@ WF_EX = models.WorkflowExecution(
     spec={'name': 'some'},
     state=states.RUNNING,
     state_info=None,
+    context={},
     input={'foo': 'bar'},
     output={},
     params={'env': {'k1': 'abc'}},
@@ -79,6 +81,7 @@ SUB_WF_EX = models.WorkflowExecution(
     spec={'name': 'some'},
     state=states.RUNNING,
     state_info=None,
+    context={},
     input={'foo': 'bar'},
     output={},
     params={'env': {'k1': 'abc'}},
@@ -143,6 +146,14 @@ MOCK_EMPTY = mock.MagicMock(return_value=[])
 MOCK_NOT_FOUND = mock.MagicMock(side_effect=exc.DBEntityNotFoundError())
 MOCK_ACTION_EXC = mock.MagicMock(side_effect=exc.ActionException())
 
+ERROR_WF_EX = copy.deepcopy(WF_EX)
+ERROR_WF_EX['state'] = states.ERROR
+MOCK_ERROR_WF_EX = mock.MagicMock(return_value=ERROR_WF_EX)
+
+SUCCESS_WF_EX = copy.deepcopy(WF_EX)
+SUCCESS_WF_EX['state'] = states.SUCCESS
+MOCK_SUCCESS_WF_EX = mock.MagicMock(return_value=SUCCESS_WF_EX)
+
 
 @mock.patch.object(rpc_base, '_IMPL_CLIENT', mock.Mock())
 class TestExecutionsController(base.APITest):
@@ -151,7 +162,11 @@ class TestExecutionsController(base.APITest):
         resp = self.app.get('/v2/executions/123')
 
         self.assertEqual(200, resp.status_int)
-        self.assertDictEqual(WF_EX_JSON_WITH_DESC, resp.json)
+
+        expected = WF_EX_JSON_WITH_DESC.copy()
+        expected['published_global'] = '{}'
+
+        self.assertDictEqual(expected, resp.json)
 
     @mock.patch.object(db_api, 'get_workflow_execution')
     def test_get_operational_error(self, mocked_get):
@@ -164,14 +179,22 @@ class TestExecutionsController(base.APITest):
         resp = self.app.get('/v2/executions/123')
 
         self.assertEqual(200, resp.status_int)
-        self.assertDictEqual(WF_EX_JSON_WITH_DESC, resp.json)
+
+        expected = WF_EX_JSON_WITH_DESC.copy()
+        expected['published_global'] = '{}'
+
+        self.assertDictEqual(expected, resp.json)
 
     @mock.patch.object(db_api, 'get_workflow_execution', MOCK_SUB_WF_EX)
     def test_get_sub_wf_ex(self):
         resp = self.app.get('/v2/executions/123')
 
         self.assertEqual(200, resp.status_int)
-        self.assertDictEqual(SUB_WF_EX_JSON_WITH_DESC, resp.json)
+
+        expected = SUB_WF_EX_JSON_WITH_DESC.copy()
+        expected['published_global'] = '{}'
+
+        self.assertDictEqual(expected, resp.json)
 
     @mock.patch.object(db_api, 'get_workflow_execution', MOCK_NOT_FOUND)
     def test_get_not_found(self):
@@ -183,6 +206,7 @@ class TestExecutionsController(base.APITest):
                        return_value=WF_EX_WITH_PROJECT_ID)
     def test_get_within_project_id(self, mock_get):
         resp = self.app.get('/v2/executions/123', expect_errors=True)
+
         self.assertEqual(200, resp.status_int)
         self.assertTrue('project_id' in resp.json)
 
@@ -363,7 +387,11 @@ class TestExecutionsController(base.APITest):
         resp = self.app.put_json('/v2/executions/123', update_params)
 
         self.assertEqual(200, resp.status_int)
-        mock_ensure.assert_called_once_with('123')
+
+        mock_ensure.assert_called_once_with(
+            '123',
+            fields=(models.WorkflowExecution.id,)
+        )
         mock_update.assert_called_once_with('123', update_params)
 
     @mock.patch.object(
@@ -483,19 +511,16 @@ class TestExecutionsController(base.APITest):
         self.assertIn(expected_fault, resp.json['faultstring'])
 
     @mock.patch.object(rpc_clients.EngineClient, 'start_workflow')
-    @mock.patch.object(db_api, 'load_workflow_execution')
-    def test_post_auto_id(self, load_wf_ex_func, start_wf_func):
+    def test_post_auto_id(self, start_wf_func):
         # NOTE: In fact, we use "white box" testing here to understand
         # if the REST controller calls other APIs as expected. This is
         # the only way of testing available with the current testing
         # infrastructure.
-        start_wf_func.return_value = WF_EX.to_dict()
+        wf_ex_dict = WF_EX.to_dict()
+        start_wf_func.return_value = wf_ex_dict
 
         json_body = WF_EX_JSON_WITH_DESC.copy()
 
-        # We don't want to pass execution ID in this case.
-        del json_body['id']
-
         expected_json = WF_EX_JSON_WITH_DESC
 
         resp = self.app.post_json('/v2/executions', json_body)
@@ -503,15 +528,15 @@ class TestExecutionsController(base.APITest):
         self.assertEqual(201, resp.status_int)
         self.assertDictEqual(expected_json, resp.json)
 
-        load_wf_ex_func.assert_not_called()
+        kwargs = json.loads(expected_json['params'])
+        kwargs['description'] = expected_json['description']
 
         start_wf_func.assert_called_once_with(
             expected_json['workflow_id'],
             '',
-            None,
+            wf_ex_dict['id'],
             json.loads(expected_json['input']),
-            expected_json['description'],
-            **json.loads(expected_json['params'])
+            **kwargs
         )
 
     @mock.patch.object(rpc_clients.EngineClient, 'start_workflow')
@@ -540,13 +565,15 @@ class TestExecutionsController(base.APITest):
 
         load_wf_ex_func.assert_called_once_with(expected_json['id'])
 
+        kwargs = json.loads(expected_json['params'])
+        kwargs['description'] = expected_json['description']
+
         start_wf_func.assert_called_once_with(
             expected_json['workflow_id'],
             '',
             expected_json['id'],
             json.loads(expected_json['input']),
-            expected_json['description'],
-            **json.loads(expected_json['params'])
+            **kwargs
         )
 
     @mock.patch.object(rpc_clients.EngineClient, 'start_workflow')
@@ -595,12 +622,17 @@ class TestExecutionsController(base.APITest):
 
         exec_dict = source_wf_ex_json
 
+        expected_description = "{} Based on the execution '{}'".format(
+            exec_dict['description'],
+            SOURCE_WF_EX_JSON_WITH_DESC['source_execution_id']
+        )
+
         wf_exec_mock.assert_called_once_with(
             exec_dict['workflow_id'],
             '',
             exec_dict['id'],
             json.loads(exec_dict['input']),
-            exec_dict['description'],
+            description=expected_description,
             **json.loads(exec_dict['params'])
         )
 
@@ -608,10 +640,8 @@ class TestExecutionsController(base.APITest):
     @mock.patch.object(rpc_clients.EngineClient, 'start_workflow')
     def test_post_with_src_exec_id_without_exec_id(self, wf_exec_mock):
         source_wf_ex = copy.copy(SOURCE_WF_EX)
-        source_wf_ex.id = ""
 
         source_wf_ex_json = copy.copy(SOURCE_WF_EX_JSON_WITH_DESC)
-        source_wf_ex_json['id'] = ''
 
         wf_exec_mock.return_value = source_wf_ex.to_dict()
 
@@ -624,12 +654,17 @@ class TestExecutionsController(base.APITest):
 
         exec_dict = source_wf_ex_json
 
+        expected_description = "{} Based on the execution '{}'".format(
+            exec_dict['description'],
+            SOURCE_WF_EX_JSON_WITH_DESC['source_execution_id']
+        )
+
         wf_exec_mock.assert_called_once_with(
             exec_dict['workflow_id'],
             '',
-            '',
+            exec_dict['id'],
             json.loads(exec_dict['input']),
-            exec_dict['description'],
+            description=expected_description,
             **json.loads(exec_dict['params'])
         )
 
@@ -659,10 +694,27 @@ class TestExecutionsController(base.APITest):
             '',
             exec_dict['id'],
             json.loads(exec_dict['input']),
-            exec_dict['description'],
+            description=exec_dict['description'],
             **json.loads(exec_dict['params'])
         )
 
+    @mock.patch.object(rpc_clients.EngineClient, 'start_workflow')
+    def test_post_with_params_none(self, start_wf_func):
+        wf_ex_dict = WF_EX.to_dict()
+
+        start_wf_func.return_value = wf_ex_dict
+
+        json_body = WF_EX_JSON_WITH_DESC.copy()
+
+        json_body['params'] = None
+
+        expected_json = WF_EX_JSON_WITH_DESC
+
+        resp = self.app.post_json('/v2/executions', json_body)
+
+        self.assertEqual(201, resp.status_int)
+        self.assertDictEqual(expected_json, resp.json)
+
     @mock.patch.object(
         rpc_clients.EngineClient,
         'start_workflow',
@@ -688,8 +740,40 @@ class TestExecutionsController(base.APITest):
 
         self.assertIn('Bad response: 400', context.args[0])
 
-    @mock.patch.object(db_api, 'delete_workflow_execution', MOCK_DELETE)
-    def test_delete(self):
+    @mock.patch.object(
+        db_api,
+        'get_workflow_execution',
+        mock.MagicMock(return_value=(states.RUNNING,))
+    )
+    def test_delete_running_execution(self):
+        resp = self.app.delete('/v2/executions/123', expect_errors=True)
+
+        self.assertEqual(403, resp.status_int)
+        self.assertIn(
+            "Only completed executions can be deleted. "
+            "Use --force to override this. "
+            "Execution 123 is in RUNNING state",
+            resp.body.decode()
+        )
+
+    @mock.patch.object(db_api,
+                       'get_workflow_execution',
+                       mock.MagicMock(return_value=(states.ERROR,)))
+    @mock.patch.object(db_api,
+                       'delete_workflow_execution',
+                       MOCK_DELETE)
+    def test_delete_error_exec(self):
+        resp = self.app.delete('/v2/executions/123')
+
+        self.assertEqual(204, resp.status_int)
+
+    @mock.patch.object(db_api,
+                       'get_workflow_execution',
+                       mock.MagicMock(return_value=(states.SUCCESS,)))
+    @mock.patch.object(db_api,
+                       'delete_workflow_execution',
+                       MOCK_DELETE)
+    def test_delete_success_exec(self):
         resp = self.app.delete('/v2/executions/123')
 
         self.assertEqual(204, resp.status_int)
@@ -829,7 +913,7 @@ class TestExecutionsController(base.APITest):
         resource_function = kwargs['resource_function']
 
         self.assertEqual(
-            execution._get_workflow_execution_resource,
+            execution._get_workflow_execution_resource_with_output,
             resource_function
         )
 
@@ -843,7 +927,10 @@ class TestExecutionsController(base.APITest):
         args, kwargs = mock_get_all.call_args
         resource_function = kwargs['resource_function']
 
-        self.assertIsNone(resource_function)
+        self.assertEqual(
+            execution._get_workflow_execution_resource,
+            resource_function
+        )
 
     @mock.patch('mistral.db.v2.api.get_workflow_executions')
     @mock.patch('mistral.context.MistralContext.from_environ')
