@@ -20,6 +20,7 @@ from oslo_config import cfg
 from mistral.db.v2 import api as db_api
 from mistral import exceptions as exc
 from mistral.lang import parser as spec_parser
+from mistral.scheduler import base as sched_base
 from mistral.services import workflows as wf_service
 from mistral.tests.unit.engine import base
 from mistral.workflow import states
@@ -90,7 +91,7 @@ class DirectWorkflowEngineTest(base.EngineTestCase):
         self.await_task_success(task1.id)
         self.await_task_success(task2.id)
 
-        self.assertTrue(wf_ex.state, states.ERROR)
+        self.assertEqual(wf_ex.state, states.SUCCESS)
 
     def test_condition_transition_not_triggering(self):
         wf_text = """---
@@ -131,7 +132,7 @@ class DirectWorkflowEngineTest(base.EngineTestCase):
 
         self.await_task_error(task1.id)
 
-        self.assertTrue(wf_ex.state, states.ERROR)
+        self.assertEqual(wf_ex.state, states.ERROR)
 
     def test_change_state_after_success(self):
         wf_text = """
@@ -223,7 +224,7 @@ class DirectWorkflowEngineTest(base.EngineTestCase):
         wf_ex = self._run_workflow(wf_text)
 
         self.assertIn('Invalid input', wf_ex.state_info)
-        self.assertTrue(wf_ex.state, states.ERROR)
+        self.assertEqual(wf_ex.state, states.ERROR)
 
     def test_wrong_first_task_input(self):
         wf_text = """
@@ -264,7 +265,7 @@ class DirectWorkflowEngineTest(base.EngineTestCase):
         # TODO(dzimine): Catch tasks caused error, and set them to ERROR:
         # TODO(dzimine): self.assertTrue(task_ex.state, states.ERROR)
 
-        self.assertTrue(wf_ex.state, states.ERROR)
+        self.assertEqual(wf_ex.state, states.ERROR)
         self.assertIn("Failed to find action", wf_ex.state_info)
 
     def test_wrong_action_first_task(self):
@@ -739,7 +740,7 @@ class DirectWorkflowEngineTest(base.EngineTestCase):
 
         self.assertIn("Task 'task3' not found", str(exception))
 
-    def test_delete_workflow_completion_check_on_stop(self):
+    def test_delete_workflow_integrity_check_on_stop(self):
         wf_text = """---
         version: '2.0'
 
@@ -753,20 +754,13 @@ class DirectWorkflowEngineTest(base.EngineTestCase):
 
         wf_ex = self.engine.start_workflow('wf')
 
-        calls = db_api.get_delayed_calls()
-
-        mtd_name = 'mistral.engine.workflow_handler._check_and_fix_integrity'
-
-        self._assert_single_item(calls, target_method_name=mtd_name)
-
         self.engine.stop_workflow(wf_ex.id, state=states.CANCELLED)
 
-        self._await(
-            lambda:
-            len(db_api.get_delayed_calls(target_method_name=mtd_name)) == 0
-        )
+        sched = sched_base.get_system_scheduler()
 
-    def test_delete_workflow_completion_on_execution_delete(self):
+        self._await(lambda: not sched.has_scheduled_jobs())
+
+    def test_delete_workflow_integrity_check_on_execution_delete(self):
         wf_text = """---
         version: '2.0'
 
@@ -780,18 +774,11 @@ class DirectWorkflowEngineTest(base.EngineTestCase):
 
         wf_ex = self.engine.start_workflow('wf')
 
-        calls = db_api.get_delayed_calls()
-
-        mtd_name = 'mistral.engine.workflow_handler._check_and_fix_integrity'
-
-        self._assert_single_item(calls, target_method_name=mtd_name)
-
         db_api.delete_workflow_execution(wf_ex.id)
 
-        self._await(
-            lambda:
-            len(db_api.get_delayed_calls(target_method_name=mtd_name)) == 0
-        )
+        sched = sched_base.get_system_scheduler()
+
+        self._await(lambda: not sched.has_scheduled_jobs())
 
     def test_output(self):
         wf_text = """---
@@ -1133,3 +1120,22 @@ class DirectWorkflowEngineTest(base.EngineTestCase):
             # converted into a string no matter what we tried to assign. But
             # that didn't happen on Python 2.7 which caused an SQL exception.
             self.assertIsInstance(task_ex.state_info, six.string_types)
+
+    def test_single_fail_with_next_noop(self):
+        wf_text = """---
+        version: '2.0'
+
+        wf:
+          tasks:
+            task1:
+              action: std.fail
+              on-error:
+                - noop
+        """
+
+        wf_service.create_workflows(wf_text)
+
+        # Start workflow.
+        wf_ex = self.engine.start_workflow('wf')
+
+        self.await_workflow_success(wf_ex.id)
