@@ -1227,12 +1227,10 @@ class PoliciesTest(base.EngineTestCase):
 
         self.assertDictEqual({'result': 'mocked result'}, wf_output)
 
-    def test_retry_failed_join_task(self):
+    def test_retry_join_task_after_failed_task(self):
         retry_wb = """---
         version: '2.0'
-
         name: wb
-
         workflows:
           wf1:
             task-defaults:
@@ -1241,13 +1239,11 @@ class PoliciesTest(base.EngineTestCase):
                 delay: 0
             tasks:
               task1:
-                action: std.noop
                 on-success: join_task
               task2:
                 action: std.fail
                 on-success: join_task
               join_task:
-                action: std.noop
                 join: all
         """
         wb_service.create_workbook_v2(retry_wb)
@@ -1260,15 +1256,46 @@ class PoliciesTest(base.EngineTestCase):
         with db_api.transaction():
             # Note: We need to reread execution to access related tasks.
             wf_ex = db_api.get_workflow_execution(wf_ex.id)
-
             tasks = wf_ex.task_executions
 
-        self._assert_single_item(
-            tasks, name="task2", state=states.ERROR
-        )
-        self._assert_single_item(
-            tasks, name="join_task", state=states.ERROR
-        )
+        self._assert_single_item(tasks, name="task2", state=states.ERROR)
+        self._assert_single_item(tasks, name="join_task", state=states.ERROR)
+
+    def test_retry_join_task_after_idle_task(self):
+        retry_wb = """---
+        version: '2.0'
+        name: wb
+        workflows:
+          wf1:
+            task-defaults:
+              retry:
+                count: 1
+                delay: 0
+            tasks:
+              task1:
+                on-success: join_task
+              task2:
+                action: std.fail
+                on-success: task3
+              task3:
+                on-success: join_task
+              join_task:
+                join: all
+        """
+        wb_service.create_workbook_v2(retry_wb)
+
+        # Start workflow.
+        wf_ex = self.engine.start_workflow('wb.wf1')
+
+        self.await_workflow_error(wf_ex.id)
+
+        with db_api.transaction():
+            # Note: We need to reread execution to access related tasks.
+            wf_ex = db_api.get_workflow_execution(wf_ex.id)
+            tasks = wf_ex.task_executions
+
+        self._assert_single_item(tasks, name="task2", state=states.ERROR)
+        self._assert_single_item(tasks, name="join_task", state=states.ERROR)
 
     @mock.patch.object(
         std_actions.EchoAction,
@@ -1847,3 +1874,154 @@ class PoliciesTest(base.EngineTestCase):
             {},
             fail_task_ex.runtime_context["retry_task_policy"]
         )
+
+    def test_fail_on_true_condition(self):
+        retry_wb = """---
+        version: '2.0'
+
+        name: wb
+
+        workflows:
+          wf1:
+            tasks:
+              task1:
+                action: std.echo output=4
+                fail-on: <% task(task1).result <= 4 %>
+        """
+        wb_service.create_workbook_v2(retry_wb)
+
+        # Start workflow.
+        wf_ex = self.engine.start_workflow('wb.wf1')
+        self.await_workflow_error(wf_ex.id)
+
+        with db_api.transaction():
+            wf_ex = db_api.get_workflow_execution(wf_ex.id)
+            task_ex = wf_ex.task_executions[0]
+
+        self.assertEqual(task_ex.state, states.ERROR, "Check task state")
+
+    def test_fail_on_false_condition(self):
+        retry_wb = """---
+        version: '2.0'
+
+        name: wb
+
+        workflows:
+          wf1:
+            tasks:
+              task1:
+                action: std.echo output=4
+                fail-on: <% task(task1).result != 4 %>
+        """
+        wb_service.create_workbook_v2(retry_wb)
+
+        # Start workflow.
+        wf_ex = self.engine.start_workflow('wb.wf1')
+        self.await_workflow_success(wf_ex.id)
+
+        with db_api.transaction():
+            wf_ex = db_api.get_workflow_execution(wf_ex.id)
+            task_ex = wf_ex.task_executions[0]
+
+        self.assertEqual(task_ex.state, states.SUCCESS, "Check task state")
+
+    def test_fail_on_true_condition_task_defaults(self):
+        retry_wb = """---
+        version: '2.0'
+
+        name: wb
+
+        workflows:
+          wf1:
+            task-defaults:
+              fail-on: <% task().result <= 4 %>
+            tasks:
+              task1:
+                action: std.echo output=4
+        """
+        wb_service.create_workbook_v2(retry_wb)
+
+        # Start workflow.
+        wf_ex = self.engine.start_workflow('wb.wf1')
+        self.await_workflow_error(wf_ex.id)
+
+        with db_api.transaction():
+            wf_ex = db_api.get_workflow_execution(wf_ex.id)
+            task_ex = wf_ex.task_executions[0]
+
+        self.assertEqual(task_ex.state, states.ERROR, "Check task state")
+
+    @mock.patch.object(
+        std_actions.EchoAction,
+        'run',
+        mock.Mock(side_effect=[1, 2, 3, 4])
+    )
+    def test_fail_on_with_retry(self):
+        retry_wb = """---
+        version: '2.0'
+
+        name: wb
+
+        workflows:
+          wf1:
+            tasks:
+              task1:
+                action: std.echo output="mocked"
+                fail-on: <% task(task1).result <= 2 %>
+                retry:
+                  count: 3
+                  delay: 0
+        """
+        wb_service.create_workbook_v2(retry_wb)
+
+        # Start workflow.
+        wf_ex = self.engine.start_workflow('wb.wf1')
+        self.await_workflow_success(wf_ex.id)
+
+        with db_api.transaction():
+            wf_ex = db_api.get_workflow_execution(wf_ex.id)
+            task_ex = wf_ex.task_executions[0]
+
+        self.assertEqual(task_ex.state, states.SUCCESS, "Check task state")
+        self.assertEqual(
+            2,
+            task_ex.runtime_context['retry_task_policy']['retry_no']
+        )
+
+    @mock.patch.object(
+        std_actions.EchoAction,
+        'run',
+        mock.Mock(side_effect=[1, 2, 3, 4])
+    )
+    def test_fail_on_with_retry_and_with_items(self):
+        retry_wb = """---
+        version: '2.0'
+
+        name: wb
+
+        workflows:
+          wf1:
+            tasks:
+              task1:
+                with-items: x in [1, 2]
+                action: std.echo output="mocked"
+                fail-on: <% not task(task1).result.contains(4) %>
+                retry:
+                  count: 3
+                  delay: 0
+        """
+        wb_service.create_workbook_v2(retry_wb)
+
+        # Start workflow.
+        wf_ex = self.engine.start_workflow('wb.wf1')
+        self.await_workflow_success(wf_ex.id)
+
+        with db_api.transaction():
+            wf_ex = db_api.get_workflow_execution(wf_ex.id)
+            task_ex = wf_ex.task_executions[0]
+
+        self.assertEqual(task_ex.state, states.SUCCESS, "Check task state")
+        self.assertEqual(
+            1,
+            task_ex.runtime_context['retry_task_policy']['retry_no']
+        )
