@@ -19,7 +19,7 @@ import yaml
 from mistral import exceptions as exc
 from mistral.lang import parser as spec_parser
 from mistral.tests.unit import base
-from mistral import utils
+from mistral_lib import utils
 
 
 class WorkflowSpecValidationTestCase(base.BaseTest):
