@@ -20,11 +20,12 @@ from oslo_config import cfg
 from mistral.db.v2.sqlalchemy import api as db_api
 from mistral import exceptions as exc
 from mistral.lang import parser as spec_parser
-import mistral.lang.v2.tasks as tasks
+from mistral.lang.v2 import tasks
+from mistral.lang.v2 import workflows
 from mistral.services import workflows as wf_service
 from mistral.tests.unit import base
-from mistral import utils
 from mistral.workflow import states
+from mistral_lib import utils
 
 
 # Use the set_default method to set value otherwise in certain test cases
@@ -85,8 +86,7 @@ WORKFLOW_WITH_VAR_TASK_NAME = """
 ---
 version: '2.0'
 
-list_servers:
-
+engine_command_{task_name}:
   tasks:
     {task_name}:
       action: nova.servers_list
@@ -99,12 +99,51 @@ INVALID_WORKFLOW = """
 verstion: '2.0'
 
 wf:
-  type: direct
   tasks:
     task1:
       action: std.echo output="Task 1"
 """
 
+INVALID_WORKFLOW_1 = """
+---
+version: '2.0'
+
+wf:
+  tasks:
+    task1:
+      action: std.noop
+      on-success: task2 # The task "task2" doesn't exist.
+
+    task3:
+      action: std.noop
+"""
+
+WORKFLOW_WITH_LONG_TASK_NAME = """
+---
+version: '2.0'
+
+test_workflow:
+  tasks:
+    {long_task_name}:
+      action: std.noop
+
+"""
+
+WORKFLOW_WITH_LONG_JOIN_TASK_NAME = """
+---
+version: '2.0'
+
+test_workflow:
+  tasks:
+    task1:
+      on-success:
+        - {long_task_name}
+
+    {long_task_name}:
+      join: all
+
+"""
+
 
 class WorkflowServiceTest(base.DbTestCase):
     def test_create_workflows(self):
@@ -127,15 +166,14 @@ class WorkflowServiceTest(base.DbTestCase):
         self.assertEqual('wf2', wf2_spec.get_name())
         self.assertEqual('direct', wf2_spec.get_type())
 
-    def test_invalid_task_name(self):
-        for name in tasks.RESERVED_TASK_NAMES:
-            wf = WORKFLOW_WITH_VAR_TASK_NAME.format(task_name=name)
+    def test_engine_commands_are_valid_task_names(self):
+        for name in workflows.ENGINE_COMMANDS:
+            wf_text = WORKFLOW_WITH_VAR_TASK_NAME.format(task_name=name)
 
-            self.assertRaises(
-                exc.InvalidModelException,
-                wf_service.create_workflows,
-                wf
-            )
+            wf_defs = wf_service.create_workflows(wf_text)
+
+            self.assertIsNotNone(wf_defs)
+            self.assertEqual(1, len(wf_defs))
 
     def test_update_workflows(self):
         db_wfs = wf_service.create_workflows(WORKFLOW_LIST)
@@ -232,7 +270,6 @@ class WorkflowServiceTest(base.DbTestCase):
                 )
 
                 self.assertDictEqual(update_env, updated.params['env'])
-                self.assertDictEqual(update_env, updated.context['__env'])
 
                 fetched = db_api.get_workflow_execution(created.id)
 
@@ -280,14 +317,112 @@ class WorkflowServiceTest(base.DbTestCase):
                     update_env
                 )
 
-            fetched = db_api.get_workflow_execution(created.id)
+                fetched = db_api.get_workflow_execution(created.id)
 
-            self.assertDictEqual(
-                wf_exec['params']['env'],
-                fetched.params['env']
-            )
+                self.assertDictEqual(
+                    wf_exec['params']['env'],
+                    fetched.params['env']
+                )
 
-            self.assertDictEqual(
-                wf_exec['context']['__env'],
-                fetched.context['__env']
-            )
+                self.assertDictEqual(
+                    wf_exec['context']['__env'],
+                    fetched.context['__env']
+                )
+
+    def test_with_long_task_name(self):
+        long_task_name = utils.generate_string(tasks.MAX_LENGTH_TASK_NAME + 1)
+
+        workflow = WORKFLOW_WITH_LONG_TASK_NAME.format(
+            long_task_name=long_task_name
+        )
+
+        self.assertRaises(
+            exc.InvalidModelException,
+            wf_service.create_workflows,
+            workflow
+        )
+
+    def test_upper_bound_length_task_name(self):
+        long_task_name = utils.generate_string(tasks.MAX_LENGTH_TASK_NAME)
+
+        wf_text = WORKFLOW_WITH_LONG_TASK_NAME.format(
+            long_task_name=long_task_name
+        )
+
+        wf_defs = wf_service.create_workflows(wf_text)
+
+        self.assertIsNotNone(wf_defs)
+        self.assertEqual(1, len(wf_defs))
+
+    def test_with_long_join_task_name(self):
+        long_task_name = utils.generate_string(
+            tasks.MAX_LENGTH_JOIN_TASK_NAME + 1
+        )
+
+        wf_text = WORKFLOW_WITH_LONG_JOIN_TASK_NAME.format(
+            long_task_name=long_task_name
+        )
+
+        self.assertRaises(
+            exc.InvalidModelException,
+            wf_service.create_workflows,
+            wf_text
+        )
+
+    def test_upper_bound_length_join_task_name(self):
+        long_task_name = utils.generate_string(tasks.MAX_LENGTH_JOIN_TASK_NAME)
+
+        wf_text = WORKFLOW_WITH_LONG_JOIN_TASK_NAME.format(
+            long_task_name=long_task_name
+        )
+
+        wf_defs = wf_service.create_workflows(wf_text)
+
+        self.assertIsNotNone(wf_defs)
+        self.assertEqual(1, len(wf_defs))
+
+    def test_validation_mode_enabled_by_default(self):
+        self.override_config('validation_mode', 'enabled', 'api')
+
+        self.assertRaises(
+            exc.InvalidModelException,
+            wf_service.create_workflows,
+            INVALID_WORKFLOW_1
+        )
+
+        wf_defs = wf_service.create_workflows(
+            INVALID_WORKFLOW_1,
+            validate=False
+        )
+
+        # The workflow is created but it will never succeed since it's broken.
+        self.assertIsNotNone(wf_defs)
+        self.assertEqual(1, len(wf_defs))
+
+    def test_validation_mode_always_enabled(self):
+        self.override_config('validation_mode', 'mandatory', 'api')
+
+        self.assertRaises(
+            exc.InvalidModelException,
+            wf_service.create_workflows,
+            INVALID_WORKFLOW_1
+        )
+
+        self.assertRaises(
+            exc.InvalidModelException,
+            wf_service.create_workflows,
+            INVALID_WORKFLOW_1,
+            validate=False
+        )
+
+    def test_validation_mode_always_disabled(self):
+        self.override_config('validation_mode', 'disabled', 'api')
+
+        wf_defs = wf_service.create_workflows(INVALID_WORKFLOW_1)
+
+        self.assertIsNotNone(wf_defs)
+        self.assertEqual(1, len(wf_defs))
+
+        db_api.delete_workflow_definition(wf_defs[0].id)
+
+        wf_service.create_workflows(INVALID_WORKFLOW_1, validate=True)
