@@ -55,35 +55,35 @@ def _determine_verify(ctx):
         return True
 
 
-def get_session_and_auth(context, **kwargs):
-    """Get session and auth parameters
+def get_session_and_auth(ctx, **kwargs):
+    """Get session and auth parameters.
 
-    :param context: action context
-    :return: dict to be used as kwargs for client serviceinitialization
+    :param ctx: action context
+    :return: dict to be used as kwargs for client service initialization
     """
 
-    if not context:
+    if not ctx:
         raise AssertionError('context is mandatory')
 
     project_endpoint = get_endpoint_for_project(**kwargs)
     endpoint = format_url(
         project_endpoint.url,
         {
-            'tenant_id': context.project_id,
-            'project_id': context.project_id
+            'tenant_id': ctx.project_id,
+            'project_id': ctx.project_id
         }
     )
 
-    auth = Token(endpoint=endpoint, token=context.auth_token)
+    auth = Token(endpoint=endpoint, token=ctx.auth_token)
 
-    auth_uri = context.auth_uri or CONF.keystone_authtoken.www_authenticate_uri
+    auth_uri = ctx.auth_uri or CONF.keystone_authtoken.www_authenticate_uri
     ks_auth = Token(
         endpoint=auth_uri,
-        token=context.auth_token
+        token=ctx.auth_token
     )
     session = ks_session.Session(
         auth=ks_auth,
-        verify=_determine_verify(context)
+        verify=_determine_verify(ctx)
     )
 
     return {
@@ -107,7 +107,7 @@ def _admin_client(trust_id=None):
             password=CONF.keystone_authtoken.admin_password,
             project_name=project_name,
             auth_url=auth_url,
-            trust_id=trust_id
+            trusts=trust_id
         )
 
         cl.management_url = auth_url
@@ -223,8 +223,12 @@ def obtain_service_catalog(ctx):
                 "'trust_id' must be provided in the admin context."
             )
 
-        trust_client = client_for_trusts(ctx.trust_id)
-        token_data = trust_client.tokens.get_token_data(
+        # trust_client = client_for_trusts(ctx.trust_id)
+        # Using trust client, it can't validate token
+        # when cron trigger running because keystone policy
+        # don't allow do this. So we need use admin client to
+        # get token data
+        token_data = _admin_client().tokens.get_token_data(
             token,
             include_catalog=True
         )
