@@ -20,13 +20,13 @@ import abc
 from oslo_log import log as logging
 from osprofiler import profiler
 
+from mistral.db.v2 import api as db_api
 from mistral import exceptions as exc
 from mistral.lang import parser as spec_parser
-from mistral import utils as u
 from mistral.workflow import commands
 from mistral.workflow import data_flow
-from mistral.workflow import lookup_utils
 from mistral.workflow import states
+from mistral_lib import utils as u
 
 
 LOG = logging.getLogger(__name__)
@@ -128,7 +128,7 @@ class WorkflowController(object):
             mistral.workflow.commands.WorkflowCommand).
         """
 
-        if self._is_paused_or_completed():
+        if self._is_completed():
             return []
 
         return self._find_next_commands(task_ex)
@@ -202,9 +202,10 @@ class WorkflowController(object):
 
         :return: True if there is one or more tasks in cancelled state.
         """
-        t_execs = lookup_utils.find_cancelled_task_executions(self.wf_ex.id)
-
-        return len(t_execs) > 0
+        return db_api.get_task_executions_count(
+            workflow_execution_id=self.wf_ex.id,
+            state=states.CANCELLED
+        ) > 0
 
     @abc.abstractmethod
     def evaluate_workflow_final_context(self):
@@ -251,15 +252,20 @@ class WorkflowController(object):
             return []
 
         # Add all tasks in IDLE state.
-        idle_tasks = lookup_utils.find_task_executions_with_state(
-            self.wf_ex.id,
-            states.IDLE
-        )
-
         return [
             commands.RunExistingTask(self.wf_ex, self.wf_spec, t)
-            for t in idle_tasks
+            for t in self._get_task_executions(state=states.IDLE)
         ]
 
+    def _is_completed(self):
+        return states.is_completed(self.wf_ex.state)
+
     def _is_paused_or_completed(self):
         return states.is_paused_or_completed(self.wf_ex.state)
+
+    def _get_task_executions(self, **kwargs):
+        return db_api.get_task_executions(
+            workflow_execution_id=self.wf_ex.id,
+            sort_keys=[],  # disable sorting
+            **kwargs
+        )
