@@ -13,8 +13,6 @@
 #    See the License for the specific language governing permissions and
 #    limitations under the License.
 
-import copy
-
 from oslo_config import cfg
 from oslo_log import log as logging
 
@@ -23,9 +21,9 @@ from mistral.db.v2.sqlalchemy import models
 from mistral import exceptions as exc
 from mistral import expressions as expr
 from mistral.lang import parser as spec_parser
-from mistral import utils
 from mistral.utils import inspect_utils
 from mistral.workflow import states
+from mistral_lib import utils
 
 LOG = logging.getLogger(__name__)
 CONF = cfg.CONF
@@ -190,7 +188,13 @@ def publish_variables(task_ex, task_spec):
 
     wf_ex = task_ex.workflow_execution
 
-    expr_ctx = ContextView(task_ex.in_context, wf_ex.context, wf_ex.input)
+    expr_ctx = ContextView(
+        get_current_task_dict(task_ex),
+        task_ex.in_context,
+        get_workflow_environment_dict(wf_ex),
+        wf_ex.context,
+        wf_ex.input
+    )
 
     if task_ex.name in expr_ctx:
         LOG.warning(
@@ -232,12 +236,21 @@ def evaluate_task_outbound_context(task_ex):
     :param task_ex: DB task.
     :return: Outbound task Data Flow context.
     """
+    # NOTE(rakhmerov): 'task_ex.in_context' has the SQLAlchemy specific
+    # type MutableDict. So we need to create a shallow copy using dict(...)
+    # initializer and use it. It's enough to be safe in order to manipulate
+    # with entries of the result dictionary, like adding more entries.
+    # However, we must not change values themselves because they are
+    # shared between the original dictionary and the newly created.
+    # It's better to avoid using the method copy.deepcopy() because on
+    # dictionaries with many entries it significantly increases memory
+    # footprint and reduces performance.
     in_context = (
-        copy.deepcopy(dict(task_ex.in_context))
-        if task_ex.in_context is not None else {}
+        dict(task_ex.in_context)
+        if getattr(task_ex, 'in_context', None) is not None else {}
     )
 
-    return utils.update_dict(in_context, task_ex.published)
+    return utils.update_dict(in_context, getattr(task_ex, 'published', {}))
 
 
 def evaluate_workflow_output(wf_ex, wf_output, ctx):
@@ -249,7 +262,12 @@ def evaluate_workflow_output(wf_ex, wf_output, ctx):
     """
 
     # Evaluate workflow 'output' clause using the final workflow context.
-    ctx_view = ContextView(ctx, wf_ex.context, wf_ex.input)
+    ctx_view = ContextView(
+        ctx,
+        get_workflow_environment_dict(wf_ex),
+        wf_ex.context,
+        wf_ex.input
+    )
 
     output = expr.evaluate_recursively(wf_output, ctx_view)
 
@@ -258,19 +276,14 @@ def evaluate_workflow_output(wf_ex, wf_output, ctx):
     return output or ctx
 
 
-def add_current_task_to_context(ctx, task_id, task_name):
-    ctx['__task_execution'] = {
-        'id': task_id,
-        'name': task_name
+def get_current_task_dict(task_ex):
+    return {
+        '__task_execution': {
+            'id': task_ex.id,
+            'name': task_ex.name
+        }
     }
 
-    return ctx
-
-
-def remove_internal_data_from_context(ctx):
-    if '__task_execution' in ctx:
-        del ctx['__task_execution']
-
 
 def add_openstack_data_to_context(wf_ex):
     wf_ex.context = wf_ex.context or {}
@@ -288,30 +301,16 @@ def add_execution_to_context(wf_ex):
     wf_ex.context['__execution'] = {'id': wf_ex.id}
 
 
-def add_environment_to_context(wf_ex):
-    # TODO(rakhmerov): This is redundant, we can always get env from WF params
-    wf_ex.context = wf_ex.context or {}
-
-    # If env variables are provided, add an evaluated copy into the context.
-    if 'env' in wf_ex.params:
-        env = copy.deepcopy(wf_ex.params['env'])
-
-        if ('evaluate_env' in wf_ex.params and
-                not wf_ex.params['evaluate_env']):
-            wf_ex.context['__env'] = env
-        else:
-            wf_ex.context['__env'] = expr.evaluate_recursively(
-                env,
-                {'__env': env}
-            )
-
-
 def add_workflow_variables_to_context(wf_ex, wf_spec):
     wf_ex.context = wf_ex.context or {}
 
     # The context for calculating workflow variables is workflow input
     # and other data already stored in workflow initial context.
-    ctx_view = ContextView(wf_ex.context, wf_ex.input)
+    ctx_view = ContextView(
+        get_workflow_environment_dict(wf_ex),
+        wf_ex.context,
+        wf_ex.input
+    )
 
     wf_vars = expr.evaluate_recursively(wf_spec.get_vars(), ctx_view)
 
@@ -325,3 +324,33 @@ def evaluate_object_fields(obj, context):
 
     for k, v in evaluated_fields.items():
         setattr(obj, k, v)
+
+
+def get_workflow_environment_dict(wf_ex):
+    if not wf_ex:
+        return {}
+
+    if wf_ex.root_execution_id:
+        return get_workflow_environment_dict(wf_ex.root_execution)
+
+    env_dict = wf_ex.params['env'] if 'env' in wf_ex.params else {}
+
+    return {'__env': env_dict}
+
+
+def get_workflow_execution_published_global(wf_ex):
+    res = {}
+
+    # Variables that get published globally are stored in the
+    # workflow execution "context" field. So we just need to
+    # copy its content excluding all internally used keys and
+    # workflow variables defined under "vars" section in the
+    # workflow text.
+    exclude = {'__execution', 'openstack'}
+    exclude = exclude.union(wf_ex.spec.get('vars', {}))
+
+    for k, v in wf_ex.context.items():
+        if k not in exclude:
+            res[k] = v
+
+    return res
